#include "advhiscoreswindow.h"

adv_hiscores_window::adv_hiscores_window(translation* T,skin* sk,counters_window* ctrs,
QString date_template,int max_records):
T(T),ctrs(ctrs),date_template(date_template),max_records(max_records)
{
	t.setParent(this);
	t.addTab(&g_page_global,T->str[ins_overall]);
	t.addTab(&g_page_f,T->str[ins_fl]);
	t.addTab(&g_page_nf,T->str[ins_nf]);

	reset.setText(T->str[ins_reset]);
	load.setText(T->str[ins_load_history]);
	ok.setText(T->str[ins_ok]);

	layout=new QGridLayout(this);
	layout->addWidget(&t,0,0,1,4);
	layout->addWidget(&ok,1,3,1,1);
	layout->addWidget(&reset,1,1,1,1);
	layout->addWidget(&load,1,2,1,1);

	l_global=new QVBoxLayout(&g_page_global);
	l_f=new QVBoxLayout(&g_page_f);
	l_nf=new QVBoxLayout(&g_page_nf);

	int cur=0;
	for(int i=1;i<ctrs->counters.size();++i)
		if(ctrs->counters[i] && ctrs->counters[i]->rec && ++cur<=max_records) 
			add_record(ctrs->counters[i]->get_menu(),
				ctrs->counters[i]->rec,
				ctrs->counters[i]->formula->precision);
	
	connect(&ok,SIGNAL(clicked()),this,SLOT(quit()));
	connect(&reset,SIGNAL(clicked()),this,SLOT(reseths()));
	connect(&load,SIGNAL(clicked()),this,SLOT(loadhs()));

	Qt::WindowFlags f=windowFlags();
	f|=Qt::WindowStaysOnTopHint;
	setWindowFlags(f);
	setWindowTitle(T->str[ins_High_scores]);
	setWindowIcon(sk->program_icon);
}

void adv_hiscores_window::new_record_group
(const QString& name,double* src,int pr,QString* n_src,QDateTime* d_src,record_state* has_src,QLayout* where)
{
	QLabel* l;
	QGroupBox* g_rec;
	QGridLayout* l_rec;
	QString level_names[n_stand_levels+1]={"",T->str[ins_beg],T->str[ins_int],T->str[ins_exp]};

	g_rec=new QGroupBox;
	g_rec->setTitle(name);
	l_rec=new QGridLayout(g_rec);
	l_rec->setMargin(0);
	l_rec->setSpacing(5);
	for(int i=1;i<=n_stand_levels;++i)
	{
		l=new QLabel;
		l->setText(level_names[i]);
		l_rec->addWidget(l,i,1,1,1);

		l=new QLabel;
		if(has_src[i]==rs_has_record_and_name) l->setText(QString::number(src[i],'f',pr)); 
		else l->setText("");
		l_rec->addWidget(l,i,2,1,1);

		l=new QLabel;
		if(has_src[i]==rs_has_record_and_name) l->setText(d_src[i].toString(date_template)); 
		else l->setText("");
		l_rec->addWidget(l,i,3,1,1);

		l=new QLabel;
		l->setText(n_src[i]);
		l_rec->addWidget(l,i,4,1,1);
	}
	where->addWidget(g_rec);
}

void adv_hiscores_window::add_record(const QString& name,record* r,int pr)
{
	new_record_group(name,r->global,pr,r->n_global,r->d_global,r->has_global,l_global);
	new_record_group(name,r->f,pr,r->n_f,r->d_f,r->has_f,l_f);
	new_record_group(name,r->nf,pr,r->n_nf,r->d_nf,r->has_nf,l_nf);
}


void adv_hiscores_window::quit()
{
	done(0);
}
void adv_hiscores_window::reseths()
{
	if(!ask(T->str[ins_sure])) return;
	
	for(int i=1;i<ctrs->counters.size();++i)
		if(ctrs->counters[i] && ctrs->counters[i]->rec) ctrs->counters[i]->reset_records();
	emit reset_hiscores();
	done(0);
}
void adv_hiscores_window::loadhs()
{
	emit load_hiscores();
	done(0);
}

adv_hiscores_window::~adv_hiscores_window()
{
	disconnect(&ok,SIGNAL(clicked()),this,SLOT(quit()));
	disconnect(&reset,SIGNAL(clicked()),this,SLOT(reseths()));
	disconnect(&load,SIGNAL(clicked()),this,SIGNAL(load_hiscores()));
}

