#include "board.h"

struct excprob{};

void board::el_add_event(int id)
{
	clone_events << clone_event(last_event,id);
}

void board::el_next_click(int& last_click)
{
	if(last_event-last_click > ce_hesitation_time) el_add_event(ce_hesitation);
	last_click=last_event;
}

void board::el_check_guessed_click(int x,int y)
{
	if(!game_started) return;
	if( !opened_sq_around(x,y) || el_is_possible(x,y,1) )
		el_add_event(ce_guessed_click);
}

void board::el_check_guessed_flag(int x,int y)
{
	if(!game_started || !opened_sq_around(x,y) || el_is_possible(x,y,0) )
		el_add_event(ce_guessed_flag);
}

bool board::el_is_possible(int x,int y,int mine)
{
	bool possible;
	
	pmines=0;
	p_reset_border();
	for(int i=0;i<p_brd.size();++i)
		delete p_brd[i];
	p_brd.clear();
	p_brd << new border;	// fake
	p_brd << new border;	// the border containing (x,y)
	
	process_border_c(1,x,y);
	c_brd=p_brd[1];
	c_brd->min_pmines=D.n_mines;
	c_brd->max_pmines=0;
	
	if(mine)
		squares[x][y].pmine=1;
	else
		squares[x][y].pmine=-1;
	
	try
	{
		possible=has_solutions(0);
	}
	catch(excprob)
	{
		possible=false;	// fail; we do not claim the click was guessed if we don't know for sure
	}
	
	for(int i=0;i<c_brd->c.size();++i) c_brd->c[i]->pmine=0;	// clear the border
	
	return possible;
}

bool board::has_solutions(int sq)
{
	if(++probabilities_iter>ce_iterations) throw excprob();
	if(sq==c_brd->o_x.size())
	{
		return true;
	}
	
	// determine unknown squares adjacent to c_brd->o[sq]
	square* adj[8];
	int n_adj=0;
	int x=c_brd->o_x[sq],y=c_brd->o_y[sq];
	int a_mines=squares[x][y].num-known_mines_around(x,y);	// unknown mines around square
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].unknown())
				adj[n_adj++]=&squares[i][j];
	if(n_adj<a_mines || a_mines<0) return false;	// not enough place or too many mines around

	++sq;
	if(!a_mines)
	{	// enough mines!
		for(int i=0;i<n_adj;++i) adj[i]->pmine=-1;
		return has_solutions(sq);
	}
	pmines+=a_mines;
	if(pmines>D.n_mines) 
	{		// too many mines
		pmines-=a_mines;
		return false;
	}

	// first combination
	for(int i=1;i<=a_mines;++i) adj[n_adj-i]->pmine=1;
	for(int i=0;i<n_adj-a_mines;++i) adj[i]->pmine=-1;
	while(1)
	{
		int cur,o;
		if(has_solutions(sq)) return true;
		for(cur=n_adj-1;cur>=0 && adj[cur]->pmine==-1;--cur);	// safes
		for(o=0;cur>=0 && adj[cur]->pmine==1;--cur,++o);	// mines
		if(cur<0) break;
		adj[cur]->pmine=1;adj[++cur]->pmine=-1;
		for(++cur;cur<n_adj-o+1;++cur) adj[cur]->pmine=-1;
		for(;cur<n_adj;++cur) adj[cur]->pmine=1;
	}

	// clear
	pmines-=a_mines;
	for(int i=0;i<n_adj;++i) adj[i]->pmine=0;
	
	return false;
}

void board::el_show_opening(int x,int y)
{
	QQueue< QPair<int,int> > q;
	QPair<int,int> cur_sq;
	int i,j;

	q.enqueue(qMakePair(x,y));
	squares[x][y].opened=true;
	
	while(!q.isEmpty())
	{
		cur_sq=q.dequeue();
		i=cur_sq.first;j=cur_sq.second;
		
		if(!squares[i][j].num)
		{
			for(int k=i-1;k<=i+1;++k)
				for(int l=j-1;l<=j+1;++l)
					if(squares[k][l].openable())
					{
						q.enqueue(qMakePair(k,l));
						squares[k][l].opened=true;			
					}
		}
	}
	op_hit=true;
}

void board::el_click(int x,int y)
{
	if(squares[x][y].mine)
	{
		if(luck_allowed &&
			((luck_library && apply_luck_library(x,y)) || (luck_solver && apply_luck_solver(x,y))))
		{
			el_click(x,y);
			return;
		}
		el_add_event(ce_blast);
		squares[x][y].opened=true;
	}
	else
	{
		if(!squares[x][y].num) 
		{
			el_show_opening(x,y);
			if(last_square_is_op) op_reacted=true;
			game_started=last_square_is_op=true;
		}
		else 
		{
			squares[x][y].opened=true;
			game_started=true;
			if(cur_isl && squares[x][y].island && cur_isl!=squares[x][y].island) 
				isl_changed=true;
			cur_isl=squares[x][y].island;
			if(last_square_is_op)
			{
				op_reacted=true;
				last_square_is_op=false;
			}
		}
		if(cheat_allowed && autoflag)
			for(int i=1;i<=D.width;++i)
				for(int j=1;j<=D.height;++j)
					if(squares[i][j].opened && squares[i][j].num && 
					squares[i][j].num>=closed_sq_around(i,j))
						for(int k=i-1;k<=i+1;++k)
							for(int l=j-1;l<=j+1;++l)
								if(squares[k][l].flaggable())
									squares[k][l].flagged=true;
	}
	square_opened=true;
}

void board::el_chord(int x,int y,bool odf)
{
	bool wasted=true;
	int w=odf?ce_wasted_chord_odf:ce_wasted_chord;
	if(mines_around(x,y)==flags_around(x,y) && squares[x][y].opened)
	{
		for(int i=x-1;i<=x+1;++i)
			for(int j=y-1;j<=y+1;++j)
				if(squares[x][y].mine && !squares[x][y].flagged)
					return;
		for(int i=x-1;i<=x+1;++i)
			for(int j=y-1;j<=y+1;++j)
				if(squares[i][j].openable())
				{
					wasted=false;
					el_click(i,j);
				}
				else if(squares[i][j].flagged)
				{
					clone_events[squares[i][j].premium].event_id=ce_null_event;
				}
		if(wasted) el_add_event(w);
	}
	else
	{
		el_add_event(w);
	}
}

void board::el_check_finished()
{
	bool finished=true;
	for(int i=1;i<=D.width && finished;++i)
		for(int j=1;j<=D.height;++j)
			if(!squares[i][j].opened && !squares[i][j].mine)
			{
				finished=false;break;
			}
}

void board::el_pop_board()
{
	int l=0;
	if(n_viewed_boards>=boards.size()) throw excbrd(T->str[ins_e_no_brd]);
	int w=boards[n_viewed_boards].w,h=boards[n_viewed_boards].h,n=boards[n_viewed_boards].n;
	resize_if_needed(w,h,n);
	for(int i=1,k=0;i<=D.height;++i)
		for(int j=1;j<=D.width;++j,++k)
		{
			if( (squares[j][i].mine=boards[n_viewed_boards].board[k]) ) ++l;
			squares[j][i].restart();
		}
	if(l!=n) throw excbrd(T->str[ins_e_brd_mines]);
	if(lawnmower)
		for(int i=1;i<=D.width;++i)
			for(int j=1;j<=D.height;++j)
				if(squares[i][j].mine)
					squares[i][j].flagged=true;
	init_board_basic();
	init_islands();
	++n_viewed_boards;
}

void board::make_event_list()
{
	int last_click;
	bool l,r,m,ch,odf,sh_l;
	int x,y,cx,cy;

	if(!clone_events_enabled) return;
	if(events_built) return;
	events_built=true;
	
	cout<<"making event list"<<endl;
	n_viewed_boards=0;
	last_event=last_click=0;
	cur_isl=0; 
	cx=cy=-1;
	
	clone_events.clear();
	click_events.reset();
	el_add_event(ce_null_event);
	
	game_started=sh_l=l=r=m=ch=odf=false;
	for(int i=0;i<video.size();++i)
	{
		x=video[i].x/sk->square_size;
		y=video[i].y/sk->square_size;
		last_event=video[i].timestamp;
		op_hit=op_reacted=isl_changed=square_opened=false;

		switch(video[i].id)
		{
		case me_left_press:
			click_events.add(last_event,ce_left_press);
			if(m) break;
			l=true;sh_l=false;
			if(!is_inside_board(x,y)) break;
			if(cheat_allowed && (nono || elmar))
			{
				if(r || (superclick && squares[x][y].opened))	// chord
				{
					el_next_click(last_click);
					el_chord(x,y,odf);
					ch=r;
				}
				else	// left click
				{
					el_next_click(last_click);
					if(squares[x][y].openable())
					{
						if(!n_viewed_boards) el_pop_board(); 
						// compatibility with RawVF w/o start event
						if(!squares[x][y].mine) el_check_guessed_click(x,y);
						el_click(x,y);
					}
					ch=false;
				}
			}
			break;
		case me_left_press_with_shift:
			if(m) return;
			l=sh_l=true;
			odf=false;
			if(!is_inside_board(x,y)) return;
			if(cheat_allowed && (elmar || nono))
			{
				el_next_click(last_click);
				el_chord(x,y,odf);
				l=sh_l=true;
			}
			cur_x=x;cur_y=y;
		case me_left_click:
			click_events.add(last_event,ce_left_click);
			if(!l) break;
			l=false;
			if(!is_inside_board(x,y)) break;
			if(r || sh_l || (cheat_allowed && superclick && squares[x][y].opened))	// chord
			{
				el_next_click(last_click);
				el_chord(x,y,odf);
				ch=r;
				sh_l=false;
			}
			else	// left click
			{
				el_next_click(last_click);
				if(ch) 	// rilian click
				{
					ch=false;
					el_add_event(ce_rilian_click);
					break;
				}
				if(squares[x][y].openable())
				{
					if(!n_viewed_boards) pop_board(); // compatibility with RawVF w/o start event
					if(!squares[x][y].mine) el_check_guessed_click(x,y);
					el_click(x,y);
				}
				else 
				{
					el_add_event(ce_wasted_click);
				}
				ch=false;
			}
			break;
		case me_right_press:
			click_events.add(last_event,ce_right_press);
			if(m) break;
			r=true;sh_l=false;
			if(!is_inside_board(x,y)) break;
			if(!l)
			{
				el_next_click(last_click);
				if(squares[x][y].flaggable())
				{
					odf=true;ch=false;
					if(squares[x][y].flagged)
					{
						squares[x][y].flagged=false;
						if(qmarks)
						{
							el_add_event(ce_question);
							squares[x][y].questioned=true;
						}
						if(squares[x][y].mine)
							el_add_event(ce_misunflag);
						else
							el_add_event(ce_unflag); 
					}
					else
					{
						if(!qmarks || !squares[x][y].questioned)
						{
							squares[x][y].flagged=squares[x][y].wasted_flag=true;
							if(!squares[x][y].mine) 
							{
								squares[x][y].premium=0;
								el_add_event(ce_misflag);
							}
							else
							{
								squares[x][y].premium=clone_events.size();
								el_add_event(ce_wasted_flag);
								el_check_guessed_flag(x,y);
							}
						}
						else
						{
							squares[x][y].flagged=false;
							squares[x][y].questioned=false;
						}
					}
				}
				else if(cheat_allowed && superflag && squares[x][y].opened)
				{
					if(squares[x][y].num && squares[x][y].num>=closed_sq_around(x,y))
						for(int i=x-1;i<=x+1;++i)
							for(int j=y-1;j<=y+1;++j)
								if(squares[i][j].flaggable())
									squares[i][j].flagged=true;
				}
			}	
			break;
		case me_right_click:
			click_events.add(last_event,ce_right_click);
			if(!r) break;
			sh_l=r=false;
			if(!is_inside_board(x,y)) break;
			if(l)	// chord
			{
				ch=true;
				el_next_click(last_event);
				el_chord(x,y,false);
			}
			else
			{	
				ch=false;
			}
			odf=false;
			break;
		case me_middle_press:
			click_events.add(last_event,ce_middle_press);
			l=r=odf=ch=false;
			m=true;
			break;
		case me_middle_click:
			click_events.add(last_event,ce_middle_click);
			if(!m) break;
			m=false;
			if(!is_inside_board(x,y)) break;
			el_next_click(last_event);
			el_chord(x,y,false);
			break;
		case me_mouse_move:
			if(!is_inside_board(x,y)) break;
			if(cheat_allowed && nono && l && !ch && (cx!=x || cy!=y) && !m)
			{
				if(r || sh_l || (superclick && squares[x][y].opened))	// chord
				{
					el_next_click(last_click);
					el_chord(x,y,odf);
					ch=r;
				}
				else	// left click
				{
					el_next_click(last_click);
					if(squares[x][y].openable())
					{
						if(!squares[x][y].mine) el_check_guessed_click(x,y);
						el_click(x,y);
					}
					else 
					{
						el_add_event(ce_wasted_click);
					}
					ch=false;
				}
			}
			break;
		case ge_start_board:
			el_pop_board();
			game_started=false;
			break;
		default:
			break;
		}

		// chart events
		if(op_hit) 
		{
			click_events.add(last_event,ce_opening_hit);
		}
		if(op_reacted) 
		{
			click_events.add(last_event,ce_opening_jump);
		}
		else if(isl_changed) 
		{
			click_events.add(last_opened_tm,ce_island_jump);
			click_events.add(last_event,ce_island_jump_2);
			last_opened_tm=last_click;
		}
		else if(square_opened)
			last_opened_tm=last_click;
			
		cx=x;cy=y;
	}

	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
		{
			squares[i][j].flagged=0;
			squares[i][j].opened=squares[i][j].questioned=squares[i][j].wasted_flag=false;
		}
	n_viewed_boards=0;
	cout<<"made event list"<<endl;
}

