#include "board.h"

struct excprob{};

void board::get_probabilities()
{
	int X;	// number of non-border closed unsolved squares
	
	p_reset_border();
	p_find_isolated();
	X=p_init_border();
	
	if(!p_brd.size())	// no border!
	{
		double common_prob=(double)(D.n_mines-D.known_mines)/X;
		for(int i=1;i<=D.width;++i)
			for(int j=1;j<=D.height;++j)
				if(!squares[i][j].opened && squares[i][j].probability<0)
					squares[i][j].probability=common_prob;
		return;
	}

	// find all possible solutions on border
	probabilities_iter=0;
	pmines=0;
	try
	{
		for(int cur_brd=1;cur_brd<p_brd.size();++cur_brd)
		{
			c_brd=p_brd[cur_brd];
			c_brd->min_pmines=D.n_mines;
			c_brd->max_pmines=0;
			p_solve_border(0);
		}
	}
	catch(excprob)
	{
		err(T->str[ins_e_iter]);
		return;
	}
	
	// calculate external info for each border
	int N=D.n_mines-D.known_mines;	// unknown mines
	double* coeff=new double[D.n_mines+1];
	double numer,denom;
	double common_prob=0.0;
	for(int i=1;i<p_brd.size();++i)
	{
		double j_coeff=1.0; // binom{X}{N-j} / binom{X}{N-j_min}
		
		c_brd=p_brd[i];
		p_get_ext_info(1,i,0,1);
		denom=0;

		// calculate external multiplier for each pmines 
		for(int j=c_brd->min_pmines;j<=c_brd->max_pmines;++j)
		{
			double k_coeff;
			
			k_coeff=j_coeff; // binom{X}{N-j} / binom
			coeff[j]=0;
			
			// calc N_j = \sum_k moeb[k]*\binom{X}{N-j-k} / binom{X}{N-j_min}
			for(int k=0;k<=N-j;++k)
			{
				coeff[j]+=c_brd->moeb[k]*k_coeff;
				k_coeff=k_coeff*(N-j-k)/(X-N+j+k+1);
			}
			
			denom+=coeff[j]*c_brd->mob[j];
			j_coeff=j_coeff*(N-j)/(X-N+j+1);
		}
		
		for(int k=0;k<c_brd->c.size();++k)
		{
			numer=0;
			for(int j=c_brd->min_pmines;j<=c_brd->max_pmines;++j)
				numer+=coeff[j]*c_brd->c[k]->pr[j];
			common_prob+= (c_brd->c[k]->probability=numer/denom);
			if(numer==denom)	// mine for sure
			{
				c_brd->c[k]->psolved=(c_brd->c[k]->pmine=1);
				++D.known_mines;common_prob-=1;
			}
			else if(numer==0)	// safe for sure
			{
				c_brd->c[k]->psolved=(c_brd->c[k]->pmine=-1);
			}
			
		}
	}
	delete[] coeff;

	common_prob=(D.n_mines-D.known_mines-common_prob)/X;
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(!squares[i][j].opened && squares[i][j].probability<0)
				squares[i][j].probability=common_prob;
}

void board::p_reset_border()
{
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			squares[i][j].border=0;
}

void board::p_find_isolated()
{
	// find isolated mines
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].opened && !squares[i][j].psolved)
			{
				if(!squares[i][j].num) squares[i][j].psolved=true;
				else if(squares[i][j].num>=unknown_sq_around(i,j)+known_mines_around(i,j))
					p_set_mines(i,j);
			}
	// find isolated safe squares
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].opened && !squares[i][j].psolved)
			{
				if(!squares[i][j].num) squares[i][j].psolved=true;
				else if(squares[i][j].num<=known_mines_around(i,j))
					p_set_safe(i,j);
			}
}

void board::p_set_mines(int x,int y)
{
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].unknown()) 
			{
				squares[i][j].probability=squares[i][j].pmine=1;
				squares[i][j].psolved=true;
				++D.known_mines;	
			}
	squares[x][y].psolved=true;
}
void board::p_set_safe(int x,int y)
{
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].unknown())
			{
				squares[i][j].pmine=-1;
				squares[i][j].probability=0;
			}
	squares[x][y].psolved=true;
}

int board::p_init_border()
{
	// make the border structures
	int bor=0;
	int X=0;	// number of non-border closed squares
	for(int i=0;i<p_brd.size();++i) delete p_brd[i];
	p_brd.clear();
	p_brd << new border;	// fake
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].opened && !squares[i][j].border && unknown_sq_around(i,j))
			{
				p_brd << new border;
				process_border_o(++bor,i,j);
			}
			else if(!squares[i][j].opened && !squares[i][j].border && !opened_sq_around_without_border(i,j))
			{
				++X;	// it's closed and not adjacent to border
				squares[i][j].probability=-1;
			}
			
	return X;
}

void board::process_border_o(int border,int x,int y)
{
	squares[x][y].border=border;
	p_brd[border]->o_x << x;
	p_brd[border]->o_y << y;
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(!squares[i][j].border && !squares[i][j].opened && 
				!squares[i][j].type && !squares[i][j].pmine)
			{
				process_border_c(border,i,j);
			}
}
void board::process_border_c(int border,int x,int y)
{
	squares[x][y].border=border;
	squares[x][y].pr.clear();
	p_brd[border]->c << &squares[x][y];
	squares[x][y].probability=-1;
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(!squares[i][j].border && !squares[i][j].type && squares[i][j].opened) 
			{
				process_border_o(border,i,j);
			}
}

void board::p_solve_border(int sq)
{
	if(++probabilities_iter>n_probabilities_iterations) throw excprob();
	if(sq==c_brd->o_x.size())
	{
		// add a solution
		for(int i=0;i<c_brd->c.size();++i)
			if(c_brd->c[i]->pmine==1)
				++c_brd->c[i]->pr[pmines];
		if(pmines>c_brd->max_pmines) 
			c_brd->max_pmines=pmines; 
		if(pmines<c_brd->min_pmines) 
			c_brd->min_pmines=pmines;
		++(c_brd->mob[pmines]);
		return;
	}
	
	// determine unknown squares adjacent to c_brd->o[sq]
	square* adj[8];
	int n_adj=0;
	int x=c_brd->o_x[sq],y=c_brd->o_y[sq];
	int a_mines=squares[x][y].num-known_mines_around(x,y);	// unknown mines around square
	for(int i=x-1;i<=x+1;++i)
		for(int j=y-1;j<=y+1;++j)
			if(squares[i][j].unknown())
			{
				adj[n_adj++]=&squares[i][j];
			}
	if(n_adj<a_mines || a_mines<0) return;	// not enough places or too many mines around

	++sq;
	if(!a_mines)
	{	// enough mines!
		for(int i=0;i<n_adj;++i) adj[i]->pmine=-1;
		p_solve_border(sq);
		for(int i=0;i<n_adj;++i) adj[i]->pmine=0;
		return;
	}
	pmines+=a_mines;
	if(pmines>D.n_mines) 
	{
		pmines-=a_mines;
		return;
	}	// too many mines

	// first combination
	for(int i=1;i<=a_mines;++i) adj[n_adj-i]->pmine=1;
	for(int i=0;i<n_adj-a_mines;++i) adj[i]->pmine=-1;
	while(1)
	{
		int cur,o;
		p_solve_border(sq);
		for(cur=n_adj-1;cur>=0 && adj[cur]->pmine==-1;--cur);	// safes
		for(o=0;cur>=0 && adj[cur]->pmine==1;--cur,++o);	// mines
		if(cur<0) break;
		adj[cur]->pmine=1;adj[++cur]->pmine=-1;
		for(++cur;cur<n_adj-o+1;++cur) adj[cur]->pmine=-1;
		for(;cur<n_adj;++cur) adj[cur]->pmine=1;
	}

	// clear
	pmines-=a_mines;
	for(int i=0;i<n_adj;++i) adj[i]->pmine=0;
}

void board::p_get_ext_info(int cur, int proc, int sum, int cur_num)
{// current border, excluded border, current sum, current number of solutions
	if(cur==proc)
		p_get_ext_info(cur+1,proc,sum,cur_num);
	else if(cur==p_brd.size())
		c_brd->moeb[sum]+=cur_num;
	else
		for(int i=p_brd[cur]->min_pmines;i<=p_brd[cur]->max_pmines;++i)
		{
			if(p_brd[cur]->mob[i])
				p_get_ext_info(cur+1,proc,sum+i,cur_num*p_brd[cur]->mob[i]);
		}
}

