#include "board.h"

void board::start_video()
{
	video.clear();
	video_duration=0;
	events_built=false;
}

void board::end_video()
{
	if(D.state>=playing_video) return;
	emit video_finished();
}

void board::write_preflags()
{
	for(int i=1;i<=D.width;++i)
		for(int j=1;j<=D.height;++j)
			if(squares[i][j].flagged)
			{
				write_mouse_event(me_right_press,i*sk->square_size,j*sk->square_size);
				write_mouse_event(me_right_click,i*sk->square_size,j*sk->square_size);
			}
}

void board::write_mouse_event(int event,int x,int y)
{
	video << video_event(video_duration=Timer->get_time(),event,x,y);
}

void board::write_scroll_event(int event,int x,int y)
{
	video << video_event(video_duration=Timer->get_time(),event,x,y);
}

void board::write_board_event(int event,int x,int y)
{
	video << video_event(video_duration,event,x,y);
}

void board::write_game_event(int event)
{
	video << video_event(video_duration,event,1,1);
}

void board::replace_fail_with_unblast()
{
	video_duration=video[video.size()-1].timestamp=Timer->get_time();
	video[video.size()-1].id=me_unblast;
	video[video.size()-1].x=video[video.size()-1].y=1;
}

inline QString time_to_string(int time)
{
	return QString::number(time/1000)+"."+QString::number(time%1000).rightJustified(3, QLatin1Char('0'), true);
}
void board::save_rawvf(const QString& file, gamemode mode)
{
	QDateTime d=QDateTime::currentDateTime();
	QFile rawvf;
	QString levelnames[]={"","beginner","intermediate","expert","custom","custom"};
	QString modenames[]={"classic","lucky","density","upk","cheat","edit"};

	if(!boards.size()) return;
	
	rawvf.setFileName(file);
	if(!rawvf.open(QIODevice::WriteOnly | QIODevice::Truncate)) return;
	
	rawvf.write("RawVF_Version: Rev3\n");
	rawvf.write("Program: FreeSweeper\n");
	rawvf.write(QString("Version: R"+QString::number(release_id)+"\n").toUtf8());
	rawvf.write(QString("Player: "+player+"\n").toUtf8());
	rawvf.write(QString("Timestamp: "+d.toString(Qt::ISODate)+"\n").toUtf8());
	if(!marathon)
	{
		rawvf.write(QString("Level: "+levelnames[D.level]+"\n").toUtf8());
		rawvf.write(QString("Width: "+QString::number(D.width)+"\n").toUtf8());
		rawvf.write(QString("Height: "+QString::number(D.height)+"\n").toUtf8());
		rawvf.write(QString("Mines: "+QString::number(D.n_mines)+"\n").toUtf8());
	}
	else
		rawvf.write(QString("Level: marathon\n").toUtf8());
	rawvf.write(QString("Skin: "+sk->name+"\n").toUtf8());
	rawvf.write(QString("SquareSize: "+QString::number(sk->square_size)+"\n").toUtf8());
	rawvf.write(QString("Mode: "+modenames[mode]+"\n").toUtf8());
	if(qmarks) rawvf.write("QuestionMarks: on\n");
	if(luck_allowed)
	{
		if(luck_library) rawvf.write("LuckLibrary: on\n");
		if(luck_solver) rawvf.write("LuckSolver: on\n");
	}
	if(cheat_allowed) 
	{
		rawvf.write(QString("Lives: "+QString::number(lives)+"\n").toUtf8());
		if(autoflag) rawvf.write("Autoflag: on\n");
		if(lawnmower) rawvf.write("Lawnmower: on\n");
		if(elmar) rawvf.write("ElmarTechnique: on\n");
		if(nono) rawvf.write("NonoMouse: on\n");
		if(superclick) rawvf.write("SuperClick: on\n");
		if(superflag) rawvf.write("SuperFlag: on\n");
	}

	if(!marathon)
	{
		rawvf.write("Board:\n");
		for(int j=1,k=0;j<=D.height;++j)
		{
			for(int i=1;i<=D.width;++i,++k)
				if(squares[i][j].mine)
					rawvf.write("*");
				else
					rawvf.write("0");
			rawvf.write("\n");
		}
	}
	else
	{
		int s=0;
		writeln(rawvf,"NumberOfFinishedBoards: "+QString::number(marathon_description.size()));
		for(int i=0;i<marathon_description.size();++i)
		{
			writeln(rawvf,
				"FinishedBoard"+QString::number(i+1)+
					"Width: "+QString::number(marathon_description[i].width));
			writeln(rawvf,
				"FinishedBoard"+QString::number(i+1)+
					"Height: "+QString::number(marathon_description[i].height));
			writeln(rawvf,
				"FinishedBoard"+QString::number(i+1)+
					"Mines: "+QString::number(marathon_description[i].mines));
		}
		rawvf.write("Board:\n");
		while(1)
		{
			writeln(rawvf,QString::number(boards[s].w));
			writeln(rawvf,QString::number(boards[s].h));
			writeln(rawvf,QString::number(boards[s].n));
			for(int j=1,k=0;j<=boards[s].h;++j)
			{
				for(int i=1;i<=boards[s].w;++i,++k)
					if(boards[s].board[k])
						rawvf.write("*");
					else
						rawvf.write("0");
				rawvf.write("\n");
			}
			++s;
			if(s<boards.size())
				rawvf.write("-\n");
			else
				break;
		}
	}
	
	// write events
	QString events[]={"lc","lr","rc","rr","mv","mc","mr","sc","lr","un"};
	QString scroll_events[]={"sx","sy"};
	QString game_events[]={"start","won","boom","odd"};
	QString bevents[]={"closed","pressed","flag","qm","pressedqm","blast",
						"number0","number1","number2","number3","number4",
						"number5","number6","number7","number8",
						};
	for(int i=0;i<video.size();++i)
	{
		if(video[i].id==ve_no_event)
			continue;
		else if(!(video[i].id & 0x1F0))	// mouse event
			writeln(rawvf, time_to_string(video[i].timestamp)+" "+
					events[video[i].id]+
					" ("+QString::number(video[i].x-sk->square_size)+" "+	
					QString::number(video[i].y-sk->square_size)+")");
		else if(!(video[i].id & 0x1E0))	// scroll event
			writeln(rawvf, time_to_string(video[i].timestamp)+" "+
					scroll_events[video[i].id & ~0x1F0]+
					" "+QString::number(video[i].x)+" "+	
					QString::number(video[i].y));
		else if(!(video[i].id & 0x100))	// game event
		{
			writeln(rawvf,time_to_string(video[i].timestamp)+" "+game_events[video[i].id & ~0xF0]);
		}
		else	// board event
			rawvf.write(QString(
				bevents[video[i].id & ~0x100]+
				" "+QString::number(video[i].x)+" "+QString::number(video[i].y)+"\n"
					).toUtf8());
	}

	rawvf.close();
}

void board::load_rawvf(const QString& filename)
{
	int w,h,n;
	int intmode;
	double coeff=1.0;
	QString opt="",val;
	QMap<QString,QString> options;
	QString modenames[]={"classic","lucky","density","upk","cheat"};
	
	cout<<"Loading RawVF "<<filename<<endl;
	clear_boards();
	n_finished_boards=n_viewed_boards=0;
	
	QFile rawvf;
	rawvf.setFileName(filename);
	if(!rawvf.open(QIODevice::ReadOnly))
	{
		throw excvid(T->str[ins_e_open_vid]);
		return;
	}
	
	mouse_path.clear();
	// read options
	while(opt!="board")
	{
		QString line;
		int sp;

		if(rawvf.atEnd()) throw excvid(T->str[ins_e_no_brd]);

		line=QString::fromUtf8(rawvf.readLine());
		// if it's not a valid entry ("Name = Value"), ignore
		if( (sp=line.indexOf(':')) == -1) continue;	
		opt=line.left(sp).trimmed().toLower();
		val=line.mid(sp+1).trimmed();
		if(val=="") continue;
		options[opt]=val;	
	}

	int sq_size=options["squaresize"].toInt();
	if(!sq_size || sq_size<0) sq_size=sk->standard_square_size;
	if(sq_size!=sk->square_size) 
		coeff=sk->square_size/(double)sq_size;
		
	// read mode & options
	intmode=-1;
	for(int i=0;i<n_modes;++i)
		if(options["mode"]==modenames[i])
		{
			intmode=i;
			break;
		}
	if(intmode<0) intmode=cheat;
	qmarks=options["questionmarks"]=="on";
	emit video_mode((gamemode)(intmode%n_modes));
	
	// read player id
	val=options["player"];
	emit display_player(val);
	if(!cheat_allowed)
	{
		lives=1;
		autoflag=lawnmower=elmar=nono=false;
	}
	else
	{
		lives=options["continues"].toInt();
		autoflag=options["autoflag"]=="on";
		lawnmower=options["lawnmower"]=="on";
		elmar=options["elmartechnique"]=="on";
		nono=options["nonomouse"]=="on";
		superflag=options["superflag"]=="on";
		superclick=options["superclick"]=="on";
	}
	if(luck_allowed)
	{	
		luck_library=options["lucklibrary"]=="on";
		luck_solver=options["lucksolver"]=="on";
	}
	
	marathon=options["level"]=="marathon";
	
	if(!marathon)
	{
		// initialize board
		// read size and mine number
		w=options["width"].toInt();
		h=options["height"].toInt();
		n=options["mines"].toInt();	
		resize_if_needed(w,h,n);
		clear_board();

		// place mines
		for(int j=1;j<=h;++j)
		{
			for(int i=1;i<=w;++i)
			{
				char c;
				rawvf.getChar(&c);
				squares[i][j].mine=(c=='*');
			}
			rawvf.readLine();
		}
		push_board();
	}
	else
	{
		QString line="";
		int num_of_boards=options["numberoffinishedboards"].toInt();
		if(num_of_boards<=0) throw excvid("invalid number of boards in the marathon");
		marathon_description.clear();
		for(int i=0;i<num_of_boards;++i)
		{
			marathon_description << board_description
					(options["finishedboard"+QString::number(i+1)+"width"].toInt(),
					options["finishedboard"+QString::number(i+1)+"height"].toInt(),
					options["finishedboard"+QString::number(i+1)+"mines"].toInt());
		}
		while(1)
		{
			w=readln(rawvf).toInt();
			h=readln(rawvf).toInt();
			n=readln(rawvf).toInt();	
			board_layout bds(w,h,n);
			
			// place mines
			for(int j=1,k=0;j<=h;++j)
			{
				for(int i=1;i<=w;++i,++k)
				{
					char c;
					rawvf.getChar(&c);
					bds.board[k]=(c=='*');
				}
				rawvf.readLine();
			}
			boards << bds;
			
			line=readln(rawvf);
			if(line!="-")
			{
				video << video_event(0,ge_start_board,0,0);
				break;
			}
		}
	}

	// read events
	int ev_tm=0,ev_x,ev_y;
	videoeventid ev;
	video_duration=0;
	video.clear();
	while(!rawvf.atEnd())
	{
		QByteArray line;
		line=rawvf.readLine();
		
		if(line.size()<=2 ||
			line[0]=='c' || line[0]=='f' || line[0]=='q' || line[0]=='p' || line[0]=='n') 	// board event
		{
			continue;
		}
		else if(line[0]=='s') // start
		{
			video << video_event(ev_tm,ge_start_board,0,0);
		}
		else if(line[0]=='w')	// won
		{
			video << video_event(ev_tm,ge_term_won,0,0);
		}
		else if(line[0]=='b' && line[1]=='o')	// lost
		{
			video << video_event(ev_tm,ge_term_lost,0,0);
		}
		else if(line[0]=='o')	// odd termination
		{
			video << video_event(ev_tm,ge_term_weird,0,0);
		}
		else if(line[0]=='e')	// end of video
		{
			video << video_event(video_duration=ev_tm,ve_no_event,0,0);
			break;
		}
		else if(isdigit(line[0]) || line[0]=='-')	// mouse event
		{
			int i=(line[0]=='-'?1:0);

			ev_tm=0;
			while(line[i]!='.' && i<line.size()) ev_tm=ev_tm*10+line[i++]-'0';
			ev_tm=ev_tm*1000+(line[i+1]-'0')*100+(line[i+2]-'0')*10;
			if(isdigit(line[i+3])) ev_tm+=(line[i+3]-'0');
			while(line[++i]!=' ' && i<line.size());
			while(line[++i]==' ' && i<line.size());
			if(line[0]=='-') ev_tm=0;
			
			if(i+1>=line.size()) continue;
			if(line[i]=='l')
				if(line[i+1]=='r')
					ev=me_left_click;
				else if(line[i+1]=='c')
					ev=me_left_press;
				else
					throw excvid(T->str[ins_e_inv_ev]+" '"+line+"'");
			else if(line[i]=='r')
				if(line[i+1]=='r')
					ev=me_right_click;
				else if(line[i+1]=='c')
					ev=me_right_press;
				else
					throw excvid(T->str[ins_e_inv_ev]+" '"+line+"'");
			else if(line[i]=='m')
				if(line[i+1]=='v')
					ev=me_mouse_move;
				else if(line[i+1]=='r')
					ev=me_middle_click;
				else if(line[i+1]=='c')
					ev=me_middle_press;
				else
					throw excvid(T->str[ins_e_inv_ev]+" '"+line+"'");
			else if(line[i]=='u')
				if(line[i+1]=='n')
					ev=me_unblast;
				else
					throw excvid(T->str[ins_e_inv_ev]+" '"+line+"'");
			else if(line[i]=='s')
				if(line[i+1]=='t')
					ev=ge_start_board;
				else if(line[i+1]=='x')
					ev=se_scroll_x;
				else if(line[i+1]=='y')
					ev=se_scroll_y;
				else if(line[i+1]=='c')
					ev=me_left_press_with_shift;
				else
					throw excvid(T->str[ins_e_inv_ev]+" '"+line+"'");
			else if(line[i]=='w')
				ev=ge_term_won;
			else if(line[i]=='o')
				ev=ge_term_weird;
			else if(line[i]=='b')
				ev=ge_term_lost;
			else
			{
				throw excvid(T->str[ins_e_inv_ev]+" '"+line+"'");
			}

			if(!(ev&0x1F0))	// a click event has coordinates: read them
			{
				while(line[++i]!='(' && i<line.size());
				while(!isdigit(line[++i]) && i<line.size());
				ev_x=0;
				while(isdigit(line[i]) && i<line.size()) ev_x=ev_x*10+line[i++]-'0';
				while(!isdigit(line[++i]));
				ev_y=0;
				while(isdigit(line[i]) && i<line.size()) ev_y=ev_y*10+line[i++]-'0';
				ev_x=(int)(ev_x*coeff);
				ev_y=(int)(ev_y*coeff);
				ev_x+=sk->square_size;	// take the border into account
				ev_y+=sk->square_size;
			}
			else if(!(ev&0x1E0))
			{
				while(!isdigit(line[++i]) && i<line.size());
				ev_x=0;
				while(isdigit(line[i]) && i<line.size()) ev_x=ev_x*10+line[i++]-'0';
				while(!isdigit(line[++i]));
				ev_y=0;
				while(isdigit(line[i]) && i<line.size()) ev_y=ev_y*10+line[i++]-'0';
			}
			else
				ev_x=ev_y=sk->square_size;
				
			video << video_event(ev_tm,ev,ev_x,ev_y);
		}
	}
	if(!video_duration) video_duration=video[video.size()-1].timestamp;
	rawvf.close();
	
	events_built=false;	
	make_event_list();
	
	play_video();
}

void board::clear_boards()
{
	for(int i=0;i<boards.size();++i) boards[i].del();
	boards.clear();
	cout<<"board stack cleared"<<endl;
}

void board::push_board()
{
	int s=boards.size();
	boards << board_layout(D.width,D.height,D.n_mines);
	for(int i=1,k=0;i<=D.height;++i)
		for(int j=1;j<=D.width;++j,++k)
			boards[s].board[k]=squares[j][i].mine;
	cout<<"board pushed, stack contains "<<s+1<<" boards"<<endl;
}

void board::pop_board()
{
	int l=0;
	cout<<"board "<<n_viewed_boards<<" popped"<<endl;
	if(n_viewed_boards>=boards.size()) throw excbrd(T->str[ins_e_no_brd]);
	int w=boards[n_viewed_boards].w,h=boards[n_viewed_boards].h,n=boards[n_viewed_boards].n;
	resize_if_needed(w,h,n);
	for(int i=1,k=0;i<=D.height;++i)
		for(int j=1;j<=D.width;++j,++k)
		{
			if( (squares[j][i].mine=boards[n_viewed_boards].board[k]) ) ++l;
		}
	if(l!=n) throw excbrd(T->str[ins_e_brd_mines]);
	++n_viewed_boards;
	has_layout=true;
	initialized=has_opening_borders=false;
	restart(true);
	if(!has_opening_borders) init_opening_borders();
	set_state(playing_video);
	if(lawnmower) flag_all();
	toggle_reveal(v_show_board);
}

void board::play_video()
{
	last_square_is_op=false;
	n_finished_boards=n_viewed_boards=0;
	next_event=0;
	set_state(playing_video);
}

void board::stop_video()
{
	set_state(stopped_video);
}

void board::process_video(int time)
{
	int ev,ev_x,ev_y,ev_tm;
	cout<<"process_video "<<time<<" ("<<video_duration<<","<<next_event<<","<<(next_event<video.size()?video[next_event].timestamp:-1)<<")"<<endl;
	while(1)
	{
		if(next_event>=video.size() || next_event<0) 
		{
			stop_video();
			return;
		}

		ev_tm=video[next_event].timestamp;
		if(ev_tm>time) return;
		last_processed_event=ev_tm;
		ev=video[next_event].id;
		if(ev==ve_no_event) 
		{
			stop_video();
			return;
		}
		ev_x=video[next_event].x,ev_y=video[next_event++].y;
		
		switch(ev)
		{
		case me_left_press:
			left_press(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_left_click:case me_left_click_with_shift:
			left_click(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_right_press:
			right_press(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_right_click:
			right_click(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_middle_press:
			middle_press(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_middle_click:
			middle_click(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_mouse_move:
			mouse_move(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case me_unblast:
			restart_from_blast();
			break;
		case me_left_press_with_shift:
			left_press_with_shift(ev_x/sk->square_size,ev_y/sk->square_size,ev_x,ev_y);
			break;
		case ge_start_board:
			pop_board();
			break;
		case se_scroll_x:
			scrolling_x(ev_x);
			break;
		case se_scroll_y:
			scrolling_y(ev_y);
			break;
		default:
			break;
		}

		// no yellow arrow track
		if(!(ev & 0x1F0))
		{
			int t_x=arrow_x,t_y=arrow_y;
			arrow_x=ev_x;arrow_y=ev_y;
			repaint_arrow(t_x,t_y);
			repaint_arrow(ev_x,ev_y);
		}
	}
	if(time>video_duration) stop_video();
}

void board::path_add_left_click(int x,int y)
{
	mouse_path << path_element(x,y,pe_left_click);
}
void board::path_add_right_click(int x,int y)
{	
	mouse_path << path_element(x,y,pe_right_click);
}
void board::path_add_double_click(int x,int y)
{
	mouse_path << path_element(x,y,pe_double_click);
}
void board::path_add_mouse_move(int x,int y)
{
	if(left && right)
		mouse_path << path_element(x,y,pe_mouse_move_both);
	else if(left)
		mouse_path << path_element(x,y,pe_mouse_move_left);
	else if(right)
		mouse_path << path_element(x,y,pe_mouse_move_right);
	else
		mouse_path << path_element(x,y,pe_mouse_move);
}

void board::toggle_mouse_path(bool on)
{
	v_mouse_path=on;
	update(); 
}
void board::toggle_opening_borders(bool on)
{
	v_openings=on;
	update(); 
}
void board::toggle_reveal(bool on)
{
	v_show_board=on;
	if(on)
	{
		for(int i=1;i<=D.width;++i)
			for(int j=1;j<=D.height;++j)
				if(squares[i][j].openable())
					show_after_end(i,j);
	}
	else
	{
		for(int i=1;i<=D.width;++i)
			for(int j=1;j<=D.height;++j)
				if(squares[i][j].openable())
					squares[i][j].p=&sk->sq;
	}
	update(); 
}


