#include "clickchart.h"

clickchart::clickchart(clickevents* ce,skin* sk):ce(ce),sk(sk)
{
	len=sk->chartLength;d_len=len;cur_tm=0;
	beg=-len;
	setFixedSize((int)(len * sk->chartEWidth), sk->chartHeight);
	
	setAttribute(Qt::WA_OpaquePaintEvent);
}

int clickchart::time_to_coord(int tm)
{
	return (int)(width()*(tm-beg)/d_len);
}

void clickchart::draw_line(int beg_sq,int ind,QColor color,int height,bool spec)
{
	int tm1=ce->beg_times[ind],tm2=ce->end_times[ind];
	int c1,c2;

	if(tm1==tm2) return;
	
	if(tm1>=last_tm)
	{
		c1=time_to_coord(tm1);
		painter.drawText(c1+sk->chartLabelHOffset[0],height+sk->chartLabelVOffset[0],
			QString::number(tm1%1000/10,'g',2));
	}
	else
		c1=beg_sq;
	if(tm2<=cur_tm)
	{
		c2=time_to_coord(tm2);
		painter.drawText(c2+sk->chartLabelHOffset[1],height+sk->chartLabelVOffset[1],
			QString::number(tm2%1000/10,'g',2));
	}
	else
		c2=time_to_coord(cur_tm);
	painter.fillRect(c1,height,c2-c1, spec?sk->chartLineThicknessSpec:sk->chartLineThickness, color);
}

void clickchart::paintEvent(QPaintEvent*)
{
	int beg_sq=time_to_coord(last_tm);
	
	painter.begin(this);
	
	if(redraw)
	{
		int b=beg/1000;
		int sec=len/1000;
		
		painter.fillRect(0,0,width(),height()-1,sk->chartBackground);
		for(int i=0;i<sec;++i)
		{
			painter.setPen(sk->chartSecondFrame);
			painter.drawRect(width()*i/sec, 0, width()/sec, height());
			painter.setPen(sk->chartSecondLabel);
			if(b+i>=0) painter.drawText(width()*i/sec, height()+sk->chartSecondLabelVOffset,
				QString::number(b+i));
		}
		
		last_tm=beg;
		beg_sq=time_to_coord(beg);
	}
	
	for(int i=0;i<ce->events.size();++i)
	{
		if(ce->end_times[i]<last_tm || ce->beg_times[i]>cur_tm) continue;

		painter.setPen(sk->chartSecondLabel);		
		switch(ce->events[i])
		{
		case ce_left_press:
			draw_line(beg_sq,i,sk->chartLeft,sk->chartLeftH);
			break;
		case ce_right_press:
			draw_line(beg_sq,i,sk->chartRight,sk->chartRightH);
			break;
		case ce_middle_press:
			draw_line(beg_sq,i,sk->chartMiddle,sk->chartMiddleH);
			break;
		case ce_opening_hit:
			draw_line(beg_sq,i,sk->chartOpening,sk->chartOpeningH,true);
			break;
		case ce_island_jump:
			draw_line(beg_sq,i,sk->chartIsland,sk->chartIslandH,true);
			break;
		default:
			break;
		}
	}

	painter.end();
	redraw=true;
}

void clickchart::time_move(int tm)
{
	last_tm=cur_tm;
	cur_tm=tm;
	if(cur_tm>beg+len || cur_tm<beg || cur_tm<last_tm) 
	{
		redraw=true;
		beg=(cur_tm-len)/1000*1000+1000;
		last_tm=beg;
		repaint();
	}
	else
	{
		redraw=false;
		repaint();
	}
}

