#include "counter.h"

counter::counter():sk(0),formula(0),name(this),value(this),menu(0),type(0),row(-1),column(-1),bold(false),rec(0)
{
	setVisible(false);
}

void counter::initialize(QWidget* parent, skin* sk)
{
	this->sk=sk;

	name.setFont(sk->counter_font);
	value.setFont(sk->counter_font);

	name.setAlignment(Qt::AlignHCenter);
	value.setAlignment(Qt::AlignHCenter);

	name.setGeometry(0,0,sk->counter_width/2,sk->counter_height);
	value.setGeometry(sk->counter_width/2,0,sk->counter_width/2,sk->counter_height);
	value.setText("-");

	setParent(parent);
	menu.setParent(parent);
	menu.setCheckable(true);
}
counter::~counter()
{
	if(formula) delete formula;
	if(rec) delete rec;
}

void counter::set_name(const QString& _name)
{
	name.setText(_name);
}
void counter::set_menu(const QString& _menu)
{
	menu.setText(_menu);
}
void counter::refresh(QString _value)
{
	value.setText(_value);
}
void counter::check_boldness(double d,bool has_d)
{
	QFont f=value.font();
	bool b=has_d && ((d>bold_value) ^ bold_lower);
	if(f.bold() != b)
	{
		f.setBold(b);
		value.setFont(f);
		update();
	}
}
void counter::showEvent(QShowEvent*)
{
	menu.setChecked(true);
}
void counter::hideEvent(QHideEvent*)
{
	menu.setChecked(false);
}

QString counter::get_name()
{
	return name.text();
}

QString counter::get_menu()
{
	return menu.text();
}

QString counter::print()
{
	QString res=get_name();
	if(res!=get_menu()) res+=" ["+menu.text()+"]";
	if(type) res+=" {"+QString::number(type)+"}";
	res+=" | ";
	res+=formula->print();
	if(bold)
	{
		res+=" | b";
		if(bold_lower) res+="<"; else res+=">";
		res+=QString::number(bold_value);
	}
	if(rec)
	{
		res+=" | r";
		if(rec->lower_is_better) res+="<"; else res+=">";
		if(!rec->popup) res+="no-popup";
	}
	if(row>=0 && column>=0)
	{
		res+=" | w"+QString::number(row)+","+QString::number(column);
	}
	return res;
}


void counter::reset_records()
{
	bool l=false;
	bool popup=false;
	if(rec) 
	{
		l=rec->lower_is_better;
		popup=rec->popup;
		delete rec;
	}
	rec=new record(l,popup);
}

const QString sFPrefix="F";
const QString sNFPrefix="NF";
const QString sName="Name";
const QString sDate="Date";

void counter::save_records(settings* s)
{
	if(rec)
	{
		QString gl_s=get_menu();
		for(int j=1;j<=n_stand_levels;++j)
		{
			s->set(gl_s+QString::number(j),QString::number(rec->global[j],'f',formula->precision));
			s->set(gl_s+sFPrefix+QString::number(j),QString::number(rec->f[j],'f',formula->precision));
			s->set(gl_s+sNFPrefix+QString::number(j),QString::number(rec->nf[j],'f',formula->precision));
			s->set(gl_s+QString::number(j)+sName,rec->n_global[j]);
			s->set(gl_s+sFPrefix+QString::number(j)+sName,rec->n_f[j]);
			s->set(gl_s+sNFPrefix+QString::number(j)+sName,rec->n_nf[j]);
			s->set(gl_s+QString::number(j)+sDate,rec->d_global[j].toString());
			s->set(gl_s+sFPrefix+QString::number(j)+sDate,rec->d_f[j].toString());
			s->set(gl_s+sNFPrefix+QString::number(j)+sDate,rec->d_nf[j].toString());
		}
	}
}
void counter::load_records(settings* s)
{
	if(rec)
	{
		QString gl_s=get_menu();
		for(int j=1;j<=n_stand_levels;++j)
		{
			rec->global[j]=s->get(gl_s+QString::number(j)).toDouble();
			rec->f[j]=s->get(gl_s+sFPrefix+QString::number(j)).toDouble();
			rec->nf[j]=s->get(gl_s+sNFPrefix+QString::number(j)).toDouble();

			rec->n_global[j]=s->get(gl_s+QString::number(j)+sName);
			if(rec->n_global[j]!="") rec->has_global[j]=rs_has_record_and_name;
			rec->n_f[j]=s->get(gl_s+sFPrefix+QString::number(j)+sName);
			if(rec->n_f[j]!="") rec->has_f[j]=rs_has_record_and_name;
			rec->n_nf[j]=s->get(gl_s+sNFPrefix+QString::number(j)+sName);
			if(rec->n_nf[j]!="") rec->has_nf[j]=rs_has_record_and_name;

			rec->d_global[j]=QDateTime::fromString(s->get(gl_s+QString::number(j)+sDate));
			rec->d_f[j]=QDateTime::fromString(s->get(gl_s+sFPrefix+QString::number(j)+sDate));;
			rec->d_nf[j]=QDateTime::fromString(s->get(gl_s+sNFPrefix+QString::number(j)+sDate));
		}
	}
}

