#include "custommarathoneditor.h"

custom_marathon_editor::custom_marathon_editor(translation* T,skin* sk,QList<board_description>* current):
T(T),desc(current),edited(*desc)
{
	layout=new QGridLayout(this);
	layout->addWidget(&l_width,0,0,1,1);
	layout->addWidget(&width,0,1,1,1);
	layout->addWidget(&l_height,0,2,1,1);
	layout->addWidget(&height,0,3,1,1);
	layout->addWidget(&l_mines,0,4,1,1);
	layout->addWidget(&mines,0,5,1,1);
	layout->addWidget(&beg,1,0,1,2);
	layout->addWidget(&intr,1,2,1,2);
	layout->addWidget(&exp,1,4,1,2);
	layout->addWidget(&levels,2,0,4,6);
	layout->addWidget(&add,0,6,1,1);
	layout->addWidget(&del,1,6,1,1);
	layout->addWidget(&up,2,6,1,1);
	layout->addWidget(&down,3,6,1,1);
	layout->addWidget(&cancel,4,6,1,1);
	layout->addWidget(&ok,5,6,1,1);
	
	l_width.setText(T->str[ins_width]+":");
	l_height.setText(T->str[ins_height]+":");
	l_mines.setText(T->str[ins_mines]+":");
	add.setText(T->str[ins_add]);
	del.setText(T->str[ins_remove]);
	up.setText(T->str[ins_up]);
	down.setText(T->str[ins_down]);
	beg.setText(T->str[ins_beg]);
	intr.setText(T->str[ins_int]);
	exp.setText(T->str[ins_exp]);
	cancel.setText(T->str[ins_cancel]);
	ok.setText(T->str[ins_ok]);
	
	width.setRange(1,100);
	height.setRange(1,100);

	make_list();
	
	connect(&width,SIGNAL(valueChanged(int)),this,SLOT(adjust_mines(int)));
	connect(&height,SIGNAL(valueChanged(int)),this,SLOT(adjust_mines(int)));
	connect(&beg,SIGNAL(clicked()),this,SLOT(set_beg()));
	connect(&intr,SIGNAL(clicked()),this,SLOT(set_int()));
	connect(&exp,SIGNAL(clicked()),this,SLOT(set_exp()));
	connect(&cancel,SIGNAL(clicked()),this,SLOT(cancelled()));
	connect(&ok,SIGNAL(clicked()),this,SLOT(applied()));
	connect(&add,SIGNAL(clicked()),this,SLOT(add_board()));
	connect(&levels,SIGNAL(currentRowChanged(int)),this,SLOT(row_changed(int)));
	connect(&up,SIGNAL(clicked()),this,SLOT(go_up()));
	connect(&down,SIGNAL(clicked()),this,SLOT(go_down()));
	connect(&del,SIGNAL(clicked()),this,SLOT(del_board()));
	
	cur=0;
	set_beg();
	
	setWindowTitle(T->str[ins_cm_editor]);
	setWindowIcon(sk->program_icon);
}

void custom_marathon_editor::adjust_mines(int)
{
	mines.setRange(0,width.value()*height.value()-1);
}

void custom_marathon_editor::set_beg()
{
	width.setValue(level_width[l_beginner]);
	height.setValue(level_height[l_beginner]);
	mines.setValue(level_mines[l_beginner]);
}

void custom_marathon_editor::set_int()
{
	width.setValue(level_width[l_intermediate]);
	height.setValue(level_height[l_intermediate]);
	mines.setValue(level_mines[l_intermediate]);
}

void custom_marathon_editor::set_exp()
{
	width.setValue(level_width[l_expert]);
	height.setValue(level_height[l_expert]);
	mines.setValue(level_mines[l_expert]);
}

void custom_marathon_editor::cancelled()
{
	hide();
}

void custom_marathon_editor::applied()
{
	(*desc) = edited;
	hide();
}

void custom_marathon_editor::add_board()
{
	edited << board_description(width.value(), height.value(), mines.value());
	levels.addItem(QString::number(height.value())+"x"+QString::number(width.value())+ " / "+
					QString::number(mines.value()));
}
void custom_marathon_editor::del_board()
{
	if(cur>=0 && cur<edited.size())
	{
		edited.removeAt(cur);
		make_list();
	}
}

void custom_marathon_editor::row_changed(int row)
{
	if(row>=0) cur=row;
}

void custom_marathon_editor::make_list()
{
	levels.clear();
	
	for(int i=0;i<edited.size();++i)
		levels.addItem(QString::number(edited[i].height)+"x"+QString::number(edited[i].width)+ " / "+
					QString::number(edited[i].mines));
	levels.setCurrentRow(cur);
}

void custom_marathon_editor::go_up()
{
	if(cur<=0) return;
	board_description ld=edited[cur];
	edited[cur]=edited[cur-1];
	edited[--cur]=ld;
	make_list();
}

void custom_marathon_editor::go_down()
{
	if(cur>=edited.size()-1) return;
	board_description ld=edited[cur];
	edited[cur]=edited[cur+1];
	edited[++cur]=ld;
	make_list();
}

