#include "formulaeditor.h"

formula_editor::formula_editor(formula_editor_type type,translation* T,skin* sk,parser* p):
b_ok(T->str[ins_ok],this),b_cancel(T->str[ins_cancel],this),type(type),T(T),p(p)
{

	const int names[]={ins_time,ins_estimate,
			ins_3bv,ins_solved3bv,ins_3bvs,
			ins_zini,ins_zinis,
			ins_hzini,ins_hzinis,
			ins_ops,ins_ops_hit,
			ins_isls,ins_isls_solved,
			ins_cl,ins_cl_s,ins_w_cl,
			ins_l_cl,ins_l_cl_s,ins_w_l_cl,
			ins_r_cl,ins_r_cl_s,ins_w_r_cl,
			ins_d_cl,ins_d_cl_s,ins_w_d_cl,
			ins_cl_15,ins_cl_15_s,ins_w_cl_15,
			ins_rqp,ins_ios,
			ins_ioe,ins_thrp,ins_corr,
			ins_width,ins_height,ins_mines,
			ins_wall_mines,ins_corner_mines,
			ins_level,ins_mode,
			ins_sh_squares,ins_flags,
			ins_ones,ins_twos,ins_threes,ins_fours,ins_fives,ins_sixes,ins_sevens,ins_eights,int_numbers,
		ins_sh_ones,ins_sh_twos,ins_sh_threes,ins_sh_fours,ins_sh_fives,ins_sh_sixes,ins_sh_sevens,
			ins_sh_eights,ins_sh_numbers};

	const int values[]={f_time,f_est,
	f_3bv,f_solved_3bv,f_3bv_s,
	f_zini,f_zini_s,
	f_hzini,f_hzini_s,
	f_ops,f_ops_hit,
	f_isls,f_isls_solved,
	f_cl,f_cl_s,f_w_cl,
	f_l_cl,f_l_cl_s,f_w_l_cl,
	f_r_cl,f_r_cl_s,f_w_r_cl,
	f_d_cl,f_d_cl_s,f_w_d_cl,
	f_cl_15,f_cl_15_s,f_w_cl_15,
	f_rqp,f_ios,
	f_ioe,f_thrp,f_corr,
	f_width,f_height,f_mines,
	f_wall_mines,f_corner_mines,
	f_level,f_mode,
	f_sh_squares,f_flags,
	f_ones,f_twos,f_threes,f_fours,f_fives,f_sixes,f_sevens,f_eights,f_numbers,
	f_sh_ones,f_sh_twos,f_sh_threes,f_sh_fours,f_sh_fives,f_sh_sixes,f_sh_sevens,f_sh_eights,f_sh_numbers};

	int row[]={0,1,0,1,2,0,2,0,2,0,1,0,1,
			4,5,6,4,5,6,4,5,6,4,5,6,4,5,6,
			8,9,8,9,10,8,9,10,8,9,8,9,8,9,
			12,12,12,13,13,13,14,14,14, 12,12,12,13,13,13,14,14,14};

	int column[]={0,0,1,1,1,2,2,3,3,4,4,5,5,
			0,0,0,1,1,1,2,2,2,3,3,3,4,4,4,
			0,0,1,1,1,2,2,2,3,3,4,4,5,5,
			0,1,2,0,1,2,0,1,2,3,4,5,3,4,5,3,4,5};


	const QString names3[]={"0","1","2","3","4","5","6","7","8","9",
				"+","-","*","/","^","<",">","=","(...)","(",")","[...]","[","]",
				T->str[ins_and],T->str[ins_or]};

	const QString values3[]={"0","1","2","3","4","5","6","7","8","9",
		"+","-","*","/","^","<",">","=","","(",")","","[","]"," and "," or "};
			
	int type3[]={0,0,0,0,0,0,0,0,0,0,
			 0,0,0,0,0,1,1,1,0,0,0,1,1,1,1,1};

	int row3[]={3,0,0,0,1,1,1,2,2,2,
			0,0,1,1,2,
			0,1,2,
			2,3,3,
			0,1,2,3,3};

	int column3[]={1,0,1,2,0,1,2,0,1,2,
				4,5,4,5,4,
				7,7,7,
				5,4,5,
				8,8,8,7,8};

	l_main=new QVBoxLayout(this);
	l_1=new QGridLayout();
	l_3=new QGridLayout();
	l_4=new QGridLayout();

	f.setParent(this);
	l_main->addWidget(&f);
	l_main->setSpacing(20);
	l_1->setSpacing(0);
	l_3->setSpacing(0);
	l_4->setSpacing(0);
	l_main->addLayout(l_3);
	l_main->addLayout(l_1);
	l_main->addLayout(l_4);

	QFont font;
	font.setStyleHint(QFont::SansSerif);
	font.setPointSize(7);

	for(unsigned int i=0;i<sizeof(row)/sizeof(int);++i)
	{
		button=new QPushButton(T->str[names[i]],this);
		button->setFont(font);
		button->setFixedHeight(20);
		connect(button,SIGNAL(clicked()),&mapper,SLOT(map()));
		mapper.setMapping(button,counter_name[values[i]]);
		l_1->addWidget(button,row[i],column[i],1,1);
	}
	for(unsigned int i=0;i<sizeof(row3)/sizeof(int);++i)
	{
		button=new QPushButton(names3[i],this);
		button->setFont(font);
		button->setFixedHeight(18);
		connect(button,SIGNAL(clicked()),&mapper,SLOT(map()));
		mapper.setMapping(button,values3[i]);
		l_3->addWidget(button,row3[i],column3[i],1,1);
		if(names3[i]=="(...)") connect(button,SIGNAL(clicked()),this,SLOT(brackets()));
		if(names3[i]=="[...]") connect(button,SIGNAL(clicked()),this,SLOT(sq_brackets()));
		if(type3[i]>type) button->setEnabled(false);
	}
	for(int i=0;i<l_1->rowCount();++i)
	{
		QLabel* lbl=new QLabel("",this);
		l_1->addWidget(lbl,i,0,1,1);
		lbl->lower();
	}
	for(int i=0;i<l_3->columnCount();++i)
	{
		QLabel* lbl=new QLabel("",this);
		l_3->addWidget(lbl,0,i,1,1);
		lbl->lower();
	}

	l_4->addWidget(&b_ok,0,2,1,1);
	l_4->addWidget(&b_cancel,0,1,1,1);

	f.setAlignment(Qt::AlignRight);
	b_cancel.setShortcut(Qt::Key_Escape);

	connect(&mapper,SIGNAL(mapped(const QString&)),this,SLOT(add(const QString&)));
	connect(&b_ok,SIGNAL(clicked()),this,SLOT(ok()));
	connect(&b_cancel,SIGNAL(clicked()),this,SLOT(cancel()));
	setModal(true);

	setWindowTitle(T->str[ins_formula_editor]);
	setWindowIcon(sk->program_icon);
}

void formula_editor::add(const QString& s)
{
	f.setText(f.text()+s);
	f.setFocus();
}

void formula_editor::brackets()
{
	f.setText("( "+f.text()+" )");
	f.setFocus();
}

void formula_editor::sq_brackets()
{
	f.setText("[ "+f.text()+" ]");
	f.setFocus();
}

void formula_editor::get_formula(const QString& before)
{
	resc=0;
	f.setText(before);
	exec();
}

void formula_editor::get_formula(c_formula* before)
{
	resc=before;
	if(resc) f.setText(resc->print());
	exec();
}

void formula_editor::get_expression(formula* before)
{
	rese=before;
	if(rese) f.setText(rese->print());
	exec();
}

void formula_editor::ok()
{
	if(type==fet_counter_formula)
	{
		c_formula* after=p->parse_formula(f.text());
		if(after) 
		{ 
			resc=after;
			hide();
		}
	}
	else if(type==fet_expression)
	{
		formula* after=p->parse_expression(f.text());
		if(after) 
		{ 
			rese=after;
			hide(); 
		}
	}
}

void formula_editor::cancel()
{
	hide();
}

