#ifndef _FS_GAME_H
#define _FS_GAME_H

/* 
	This file describes the widget that implements 
	the full minesweeper game. It contains all the
	elements and manages connections between them.
*/

#include "freesweeper.h"
#include "settings.h"
#include "skin.h"
#include "board.h"
#include "display.h"
#include "smiley.h"
#include "labelid.h"
#include "timer.h"
#include "videocontrol.h"
#include "parser.h"
#include "eval.h"
#include "counterswindow.h"
#include "autosaver.h"
#include "ircmodule.h"
#include "history.h"
#include "hiscoreswindow.h"
#include "advhiscoreswindow.h"
#include "custommarathoneditor.h"
#include "optionswindow.h"

enum board_limits_type
{
	blt_imc_board_limits,blt_no_board_limits,blt_cheat_board_limits
};

class game : public QWidget
{
	Q_OBJECT
	QPainter painter;
	QSignalMapper levels_mapper;
	QSignalMapper modes_mapper;
	QSignalMapper irc_edit_mapper;
	QSignalMapper irc_history_mapper;
	
	QString F_settings;
	
	skin* Skin;
	translation* Translation;
	
	// the visible things
	board* Board;
	display* RedTimer;
	display* RedMines;
	smiley* Smiley;
	labelid* PlayerID;
	
	QScrollBar* ScrX;
	QScrollBar* ScrY;
	
	QMenuBar* MenuBar;
	QMenu *m_game,*m_options,/**m_irc,*/*m_help;
	QMenu *m_marathons,*m_irc_edit;
	QAction *m_new,/**m_restart,*m_restart_from_blast,*m_new_marathon,*/
			*m_ll,*m_ls,
			*m_preview_video,*m_read_video,*m_write_video,
			/**m_read_board,*m_write_board,*/
			*m_show_hs,*m_show_ahs,*m_show_hist,
			*m_exit_fs,
			/**m_show_opt,*m_show_cindopt,*m_show_asopt,*m_show_skinopt,
			*m_edit_m_custom,
			*m_irc_last,*m_irc_blast,*m_irc_marathon,
			*m_irc_hist_rec,*m_irc_hist_avg,*m_irc_hist_ar,
			*m_irc_hist_today_beg,*m_irc_hist_today_int,*m_irc_hist_today_exp,
			*m_irc_edit_template[n_irc_strings],
			*m_irc_copy_last,*m_irc_copy_blast,*/
			*m_about,*m_about_qt;
			
	QAction* m_level[n_game_levels+1];
	QAction* m_mode[n_modes];
	
	// subwindows
	counters_window* Counters;
	videocontrol* VideoControl;
	
	// the ticking thing
	timer* Timer;

	parser* Parser;
	evaluator* Eval;
	
	autosaver* Autosaver;
	ircmodule* IRCModule;
	
	history* History;

	QProcess history_program;

	// general settings
	QString date_template;
	
	// game options
	int level;
	
	bool hide_timer;
	bool null_mines_counter;
	bool wide_smiley_area;

	int cheat_generation_iterations;
	bool bbbv_limit_enabled;
	int cheat_bbbv_lower_limit,cheat_bbbv_upper_limit;
	bool zini_limit_enabled;
	int cheat_zini_lower_limit,cheat_zini_upper_limit;
	bool hzini_limit_enabled;
	int cheat_hzini_lower_limit,cheat_hzini_upper_limit;
	
	board_limits_type limits;
	
	bool restart_by_time;
	int restart_beg,restart_int,restart_exp;
	
	bool beep;
	int beep_beg,beep_int,beep_exp;
	
	int custom_width,custom_height,custom_mines;
	QList<board_description> custom_marathon_description;
	int marathon_length[n_stand_levels+1];
	int time_trial_length[n_stand_levels+1];
	
	// window options
	int x_hs,y_hs;	// hi-scores window coordinates
	int x_ahs,y_ahs;	// a. hi-scores window coordinates
	int x_ctr,y_ctr;	// counters window coordinates
	int x_vid,y_vid;	// video control window coordinates
	int x_ow,y_ow;		// options window
	int y_fix;	// for some reason, on some window managers qt thinks that the header isn't a part of the window
			// and QWidget::move(x(),y()) _moves_ the window down from the initial position
			// this setting is here to negate this effect
	
	// game state
	bool red_mines_state;
	bool smiley_restarts;
	bool timer_video_state;	// true iff timer ticks cause video processing
	bool counters_state;
	bool hiscores_changed;
	bool time_limits_enabled;
	
	bool video;	// true iff video is played
	bool beepped;
	bool upk_allowed;
	
	bool counters_shown;
	bool subdialogs;
	
	QString video_dir,board_dir;
	
	int records_on_ahs;
	int go_to_event_offset;
	
	bool loaded;
private:
	void draw_invisible(const QPixmap& pix);
	bool is_recognizable_filename(QString s);
public:	
	game();
	~game();

	virtual void resizeEvent(QResizeEvent* e);
	virtual void closeEvent(QCloseEvent* e);
	virtual void paintEvent(QPaintEvent* ev);	// this widget draws the top area 
							// which doesn't belong to the functional widgets
	virtual void dragEnterEvent(QDragEnterEvent* e);
	virtual void dragMoveEvent(QDragMoveEvent* e);
	virtual void dragLeaveEvent(QDragLeaveEvent* e);
	virtual void dropEvent(QDropEvent* e);
	
	virtual void mouseMoveEvent(QMouseEvent* e);
	virtual void mousePressEvent(QMouseEvent* e);
	virtual void mouseReleaseEvent(QMouseEvent* e);
	
	virtual void moveEvent(QMoveEvent* e);
	
	void init_actions();	// initialization of menu entries
	void init_menu();	// making of menus from the entries
	void init_board_options(settings& s);
	void init_general_options(settings& s);

	void init_level_actions();

	void game_won();	// process high scores and pop-ups
	void game_lost();
	QString retrieve_name(QString popup);
	
	void toggle_video_actions(bool enable);

	void toggle_custom_boards(bool allow);
	void toggle_marathons(bool allow);
	void toggle_custom_marathon(bool allow);
	void toggle_upk(bool allow);
	
	void upgrade_red_mines();
	void downgrade_red_mines();
	
	void connect_timer_to_video();
	void disconnect_timer_from_video();
	
	void show_counters();
	void hide_counters();
	
	void connect_counters();
	void disconnect_counters();
	
	void enable_time_limits();
	void disable_time_limits();
	
	void separate_counters();
	void dock_counters();
	
	void set_limits();
	void no_board_limits();
	void apply_imc_board_limits();
	void apply_cheat_board_limits();
	
	void hide_video_control();
	void show_video_control();
	
	int check_level();
	bool is_custom_level();
	
	bool is_wide_smiley(QMouseEvent* e);
	
	bool drop_file(QString path);
public slots:
	void exit_freesweeper();
	
	void place_things();
	void connect_things();
	
	void changed_state(boardstate state);
	void changed_mode(gamemode mode);
	
	void set_level(int l);

	void new_game();
	void restart();
	void restart_from_blast();
	void new_marathon();
	
	void preview_video();
	
	void load_video(QString filename="");
	void save_video(QString filename="");
	void save_secure_video(QString filename="");
	void save_rawvf_auto(QString filename);
	void save_secure_auto(QString filename);
	
	void load_board(QString filename="");
	void save_board(QString filename="");
	
	void show_hiscores();
	void show_adv_hiscores();
	void show_history(const QString& param="");
	
	void reset_time_hiscores();
	void load_records_from_history();
	void hiscores_reset();
	
	void show_options();
	void select_skin();
	void show_custom_marathon_options();
	
	void show_about_window();
	
	void irc_copy_last_finish();
	void irc_copy_last_blast();
	void irc_copy_marathon();
	
	// slots for the options window
	void wheel_as_chord();
	void wheel_as_new_game();
	
	void smiley_as_new();
	void smiley_as_restart();
	
	void adjust_to_board();
	
	void disable_id_changes();
	void enable_id_changes();

	void change_n_mines();

	void display_player(QString player);
	void change_player(QString player);

	void reset_displays();
	
	void save_video_by_default();
	
	void stop_video();
	void pause_video();
	void resume_video();
	void move_video(int time);
	
	void check_time_limits(int time);
	
	void toggle_luck_library(bool e);
	void toggle_luck_solver(bool e);
};

#endif

