#include "history.h"

const QString F_history_density="freesweeper.history-density";
const QString F_ranks="freesweeper.ranks";

inline void read2(QFile& f,char& c1,char& c2)
{
	f.getChar(&c1);f.getChar(&c2);
}

inline unsigned int read2(QFile& f)
{
	char c1,c2;
	unsigned int res;
	read2(f,c1,c2);
	res=(unsigned int)(unsigned char)c1<<8;
	res+=(unsigned int)(unsigned char)c2;
	return res;
}

history::history(translation* T,parser* p,evaluator* ev):
T(T),Parser(p),Eval(ev),ranks_changed(settings_dir!=normal_settings_dir)
{

	// initialize ranks

	QFile f(settings_dir+F_history);
	QFile fr(settings_dir+F_ranks);
	
	for(int i=0;i<=n_levels;++i) total_games[i]=0;

	// create ranks;
	for(int i=0;i<n_named_ranks;++i) id_named[i]=-1;
	if(!fr.open(QIODevice::ReadOnly))
	{
		ranks_changed=true;
		err(T->str[ins_e_open_ranks]);
	}
	else
	{
		while(!fr.atEnd())
		{
			QString rank_desc=readln_without_comments(fr).trimmed();
			int cur_id=ranks.size();
			rank* r;
			if(rank_desc=="") continue;
			r=p->parse_rank(rank_desc);
			if(!r) continue;
			
			ranks << r;
			cout<<"rank "<<r->print()<<endl;
			for(int i=0;i<n_named_ranks;++i)
				if(r->f->print()==rank_strings[i])
				{
					cout<<"named rank: name "<<i<<" id "<<cur_id<<endl;
					id_named[i]=cur_id;
					break;
				}
		}
		fr.close();
	}
	
	if(!f.open(QIODevice::ReadOnly))
	{
		err(T->str[ins_e_open_history]);
		if(!f.open(QIODevice::WriteOnly))
		{
			crit_err(T->str[ins_e_create_history]);
			QCoreApplication::exit(0);
		}
		f.close();
		if(!f.open(QIODevice::ReadOnly))
		{
			crit_err(T->str[ins_e_create_history]);
			QCoreApplication::exit(0);
		}
	}
	// fill rank trees
	board_data brd;
	gamemode mode;
	evaluator temp_ev(&brd,&mode);
	temp_ev.hist_request=true;
	while(!f.atEnd())
	{
		read_game(&brd,&mode,&temp_ev,f);
		++total_games[brd.level];
		
		for(int i=0;i<ranks.size();++i)	// ranks init
		{
			try
			{
				ranks[i]->add(brd.level,temp_ev.eval(ranks[i]->f));
			}
			catch(exceval)
			{
			}
		}
	}
	Eval->hist_request=false;
	f.close();
	
	touch_dens();
}

history::~history()
{
	for(int i=0;i<ranks.size();++i) if(ranks[i]) delete ranks[i];
}

void history::save_ranks()
{
	QFile f(settings_dir+F_ranks);

	if(!ranks_changed) return;
	if(!f.open(QIODevice::WriteOnly | QIODevice::Truncate))
		err(T->str[ins_e_open_ranks]);
	else
	{
		for(int i=0;i<ranks.size();++i)	if(ranks[i]->f) writeln(f,ranks[i]->print());
		f.close();
	}
}

void history::read_game(board_data* brd,gamemode* mode,evaluator* Eval,QFile& f)
{
	const char marker=(char)0xFF;
	int time;
	char c1,c2;
	// read block & write it to the field
	read2(f,c1,c2);
	*mode=(gamemode)(unsigned int)c1;
	brd->level=(levelname)c2;
	read2(f,c1,c2);
	time=(((unsigned int)(unsigned char)c1<<8) + (unsigned int)(unsigned char)c2)*1000;
	read2(f,c1,c2);
	time+=(((unsigned int)(unsigned char)c1<<8) + (unsigned int)(unsigned char)c2);
	brd->solved_bbbv=brd->bbbv=read2(f);
	read2(f,c1,c2);
	brd->openings=(unsigned int)(unsigned char)c1;
	brd->islands=(unsigned int)(unsigned char)c2;
	brd->zini=read2(f);
	brd->hzini=read2(f);
	brd->l_clicks=read2(f);
	brd->r_clicks=read2(f);
	brd->d_clicks=read2(f);
	brd->clicks_15=read2(f);
	brd->wasted_l_clicks=read2(f);
	brd->wasted_r_clicks=read2(f);
	brd->wasted_d_clicks=read2(f);
	brd->wasted_clicks_15=read2(f);
	brd->flags=read2(f);
	brd->unflags=read2(f);
	brd->misflags=read2(f);
	brd->misunflags=read2(f);
	brd->wasted_flags=read2(f);
	brd->distance=read2(f);
	for(int i=0;i<9;++i) brd->numbers[i]=read2(f);
	read2(f,c1,c2);
	brd->mines_in_corners=(unsigned int)(unsigned char)c1;
	brd->mines_on_walls=(unsigned int)(unsigned char)c2;
//	brd->init_obv();
	Eval->year=read2(f);
	read2(f,c1,c2);
	Eval->month=c1;
	Eval->day=c2;
	read2(f,c1,c2);
	Eval->hour=c1;
	Eval->minute=c2;
	read2(f,c1,c2);
	Eval->second=c1;
	
	do
	{
		read2(f,c1,c2);
	}
	while((c1!=marker || c2!=marker) && !f.atEnd());
		
	Eval->refresh(time);
}

inline void write2(QFile& f, int data)
{
	f.putChar(data>>8);
	f.putChar(data&0xFF);
}

void history::add(board_data* brd,gamemode mode,int time)
{
	QFile f(settings_dir+F_history);
	
	if(!f.open(QIODevice::WriteOnly | QIODevice::Append))
	{
		err(T->str[ins_e_open_history]);
		return;
	}

	f.putChar(mode);
	f.putChar(brd->level);
	++total_games[brd->level];
	
	int sec=time/1000;
	int th=time%1000;
	write2(f,sec);
	write2(f,th);
	write2(f,brd->bbbv);
	f.putChar(brd->openings);
	f.putChar(brd->islands);
	write2(f,brd->zini);
	write2(f,brd->hzini);
	write2(f,brd->l_clicks);
	write2(f,brd->r_clicks);
	write2(f,brd->d_clicks);
	write2(f,brd->clicks_15);
	write2(f,brd->wasted_l_clicks);
	write2(f,brd->wasted_r_clicks);
	write2(f,brd->wasted_d_clicks);
	write2(f,brd->wasted_clicks_15);
	write2(f,brd->flags);
	write2(f,brd->unflags);
	write2(f,brd->misflags);
	write2(f,brd->misunflags);
	write2(f,brd->wasted_flags);
	write2(f,brd->distance);
	for(int i=0;i<9;++i) write2(f,brd->numbers[i]);
	f.putChar(brd->mines_in_corners);
	f.putChar(brd->mines_on_walls);
	
	QDate d=QDate::currentDate();
	QTime t=QTime::currentTime();
	write2(f,d.year());	// year
	f.putChar(d.month()); // month
	f.putChar(d.day()); // day
	f.putChar(t.hour()); // hour
	f.putChar(t.minute()); // minute
	f.putChar(t.second()); // second
	
	f.putChar((char)0xFF);
	f.putChar((char)0xFF);
	f.putChar((char)0xFF);
	
	f.close();
	
	Eval->hist_request=true;
	for(int i=0;i<ranks.size();++i)
	{
		try
		{
			ranks[i]->add(brd->level,Eval->eval(ranks[i]->f));
			cout<<"added "<<Eval->eval(ranks[i]->f)<<" "<<ranks[i]->current<<endl;
			cout<<ranks[i]->t[brd->level].print()<<endl;
		}
		catch(exceval)
		{
			ranks[i]->current=0;
		}
	}
	Eval->hist_request=false;
}

void history::touch_dens()
{
	QFile f(settings_dir+F_history_density);
	if(!f.open(QIODevice::WriteOnly | QIODevice::Append))
	{
		err(T->str[ins_e_open_history_dens]);
		return;
	}
	f.close();
}

void history::add_dens(board_data* brd,int time,bool finished)
{
	QFile f(settings_dir+F_history_density);
	
	if(!finished && (brd->right_flags < dens_threshold*brd->n_mines)) return;
	if(!f.open(QIODevice::WriteOnly | QIODevice::Append))
	{
		err(T->str[ins_e_open_history_dens]);
		return;
	}

	f.putChar(brd->level);
	f.putChar((int)finished);
	f.putChar(brd->height);
	f.putChar(brd->width);
	write2(f,brd->n_mines);
	
	int sec=time/1000;
	int th=time%1000;
	write2(f,sec);
	write2(f,th);
	
	write2(f,brd->right_flags);
	
	QDate d=QDate::currentDate();
	QTime t=QTime::currentTime();
	write2(f,d.year());	// year
	f.putChar(d.month()); // month
	f.putChar(d.day()); // day
	f.putChar(t.hour()); // hour
	f.putChar(t.minute()); // minute
	f.putChar(t.second()); // second
	
	f.putChar((char)0xFF);
	f.putChar((char)0xFF);
	f.putChar((char)0xFF);
	
	f.close();
}

int history::get_rank(int id)
{
	if(id<0 || id>=ranks.size()) throw exceval();
	if(ranks[id]->current) return ranks[id]->current; else throw exceval();
}

int history::get_named_rank(int name)
{
	int id;
	if(name<0 || name>=n_named_ranks || id_named[name]==-1) throw exceval(); else id=id_named[name];
	return get_rank(id);
}

