#include "freesweeper.h"
#include "game.h"

const QString E_settings_dir="Couldn't create the settings directory!";
const QString box_header="FreeSweeper";
const int release_id=6;

QString settings_dir,programs_dir,normal_settings_dir;
char slash;
QTextStream cout;

int main(int argc,char** argv)
{
	QApplication app(argc,argv);

#ifdef _WIN32
	normal_settings_dir=settings_dir=QApplication::applicationDirPath()+"/";
	programs_dir=QApplication::applicationDirPath()+"/";
	slash='/';
#else
	normal_settings_dir=settings_dir=QDir::homePath()+"/.freesweeper/";
	programs_dir=QApplication::applicationDirPath()+"/";
	slash='/';
#endif
	QFile f(settings_dir+"diagnostics");
	f.open(QIODevice::WriteOnly | QIODevice::Truncate);
	cout.setDevice(&f);

	// creating the settings directory if it doesn't exist
	if(!QDir(settings_dir).exists())
	{
		cout<<"settings dir doesn't exist"<<endl;
		if(!QDir().mkdir(settings_dir))
		{
			crit_err(E_settings_dir);
			return 1;
		}
#ifndef _WIN32
		// reset the settings directory
		settings_dir="/usr/share/freesweeper/";
#endif
	}
	try
	{
#ifndef _WIN32
		if(argc>1 && argv[1]==QString("test")) return test(); 
#endif		
		game gw;
		settings_dir=normal_settings_dir;
		gw.show();
		if(argc>1) gw.drop_file(argv[1]);
		return app.exec();
	}
	catch(...)
	{
		crit_err("Something happened");
		return 1;
	}
}

void msg(const QString& s)
{
	QMessageBox::information(0,box_header,s);
}
void err(const QString& s)
{
	QMessageBox::warning(0,box_header,s);
}
bool ask(const QString& s)
{
	return (QMessageBox::question(0,box_header,s,QMessageBox::Ok|QMessageBox::Cancel,QMessageBox::Cancel)==
		QMessageBox::Ok);
}
void crit_err(const QString& s)
{
	QMessageBox::critical(0,box_header,s+"\n\n Program will be terminated.");
}

QString input(QWidget* parent,const QString& query,const QString& def)
{
	return QInputDialog::getText(parent,box_header,query,QLineEdit::Normal,def);
}

int input_int(QWidget* parent,const QString& query,int def,int min,int max)
{
	return QInputDialog::getInteger(parent,box_header,query,def,min,max);
}

void writeln(QFile& f, const QString& s)
{
	const QByteArray endline("\n");
	f.write(s.toUtf8());
	f.write(endline);
}

QString readln(QFile& f)
{
	return QString::fromUtf8(f.readLine()).remove(QChar(10)).remove(QChar(13));
}

QString readln_without_comments(QFile& f)
{
	QString s=readln(f);
	while(s.size()>=2 && s[0]=='/' && s[1]=='/' && !f.atEnd()) s=readln(f);
	return s;
}

