#include "parser.h"

parser::parser(translation* T,QString video_dir):T(T),video_dir(video_dir)
{
	for(int id=0;id<n_named_counters;++id)
		name_to_id[counter_name[id].toLower()]=(formula_id)id;
	for(int id=1;id<=n_c_formula_types;++id)
		name_to_type[c_formula_desc[id]]=(c_formula_type)id;
}

inline QCharRef parser::sym(QString& s, int cur)
{
	if(s.length()>cur) return s[cur]; else throw excparser(cur,T->str[ins_unexpected_end]);
}


void parser::ignore_spaces(QString& s,int& cur)
{
	while(s.length()>cur && s[cur].isSpace()) ++cur;
}

QString parser::get_id(QString& s,int& beg)
{
	if(s.length()<=beg || (!s[beg].isLetter() && s[beg]!='\\'))
		throw excparser(beg,T->str[ins_id_or_number]);
	int c_beg=beg;
	do
	{
		beg++;
	}
	while(s.length()>beg && (s[beg].isLetter() || s[beg]=='\\'));
	return s.mid(c_beg,beg-c_beg);
}
QString parser::get_string(QString& s,int& beg)
{
	int c_beg=beg;
	do
	{
		beg++;
	}
	while(s.length()>beg && (s[beg].isLetter() || s[beg].isDigit() || s[beg]=='\\' || s[beg]=='/' 
		|| s[beg]==':' || s[beg]=='_' || s[beg]=='.'));
	return s.mid(c_beg,beg-c_beg);
}
QString parser::get_string_with_spaces(QString& s,int& beg)
{
	int c_beg=beg;
	do
	{
		beg++;
	}
	while(s.length()>beg && (s[beg].isLetter() || s[beg].isDigit() || s[beg]=='\\' || s[beg]=='/' 
		|| s[beg]==':' || s[beg]=='_' || s[beg]=='.' || s[beg]==' '));
	return s.mid(c_beg,beg-c_beg);
}
double parser::get_number(QString& s,int& beg)
{
	double res=0.0;
	bool min=s[beg]=='-';
	if(min) ++beg;
	while(s.length()>beg && s[beg].isDigit())
	{
		res=res*10+s[beg++].digitValue();
	}
	if(s.length()<=beg) return res;
	if(s[beg]=='.')
	{
		double mult=1;
		while(s.length()>beg && s[++beg].isDigit())
		{
			mult/=10;
			res+=mult*(s[beg].digitValue());
		}
	}
	return min ? -res : res;
}
int parser::get_int(QString& s,int& beg)
{
	int res=0;
	bool min=s[beg]=='-';
	if(min) ++beg;
	while(s.length()>beg && s[beg].isDigit())
	{
		res=res*10+s[beg++].digitValue();
	}
	return min ? -res : res;
}

int parser::read_int(QString& s,int& beg)
{
	int res=get_int(s,beg);
	ignore_spaces(s,beg);
	return res;
}

int parser::get_function(QString& s,int& cur,formula*& f,formula_id x)
{
	if(s[cur]=='(')
	{
		cur=get_formula1(s,++cur,f);
		formula* ff=f;
		f=new formula(ff);
		f->id=x;
		if(s[cur]!=')') throw excparser(cur,T->str[ins_bracket_db]);
	}
	else throw excparser(cur,T->str[ins_bracket]);
	return cur;
}

formula* parser::parse_expression(QString s)
{
	int end;
	formula* f=0;
	try
	{
		s=s.toLower();
		s.replace(" and ","&");
		s.replace(" or ","|");
		end=get_expression1(s,0,f);
		if(end<s.length()) throw excparser(end,T->str[ins_garbage]);
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_expr_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
		return 0;
	}
	return f;
}
c_formula* parser::parse_formula(QString s)
{
	int end;
	c_formula* f=0;
	try
	{
		s=s.toLower();
		end=get_cf(s,0,f);
		if(end<s.length()) throw excparser(end,T->str[ins_garbage]);
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_form_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
		return 0;
	}
	return f;
}

evaluable_string* parser::parse_string(QString s)
{
	QString sep;
	c_formula* f=0;
	int cur=0,beg,sz=s.size();
	QString cur_s;
	evaluable_string* res = new evaluable_string;
	try
	{
		while(cur<sz)
		{
			beg=0;
			while(cur<sz && s[cur++] != '<') ++beg;
			if(cur==sz) ++cur;
			cur_s=s.mid(cur-beg-1,beg);
			res->strs << cur_s;
			beg=0;
			while(cur<sz && s[cur++] != '>') ++beg;
			cur_s=s.mid(cur-beg-1,beg);
			if(cur_s=="")
				f=0;
			else
			{
				cur_s=cur_s.toLower();
				get_cf(cur_s,0,f);
			}
			res->fs << f;
		}
		cout<<"string: '"<<res->print()<<"', size="<<res->fs.size()<<endl;
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_str_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
	}
	return res;
}

rank* parser::parse_rank(QString s)
{
	int end;
	formula* f=0;
	bool lower_is_better;
	try
	{
		s=s.toLower();
		end=get_formula1(s,0,f);
		find_pipe(s,end);
		ignore_spaces(s,++end);
		lower_is_better = (s.length()>end && s[end]=='<');	
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_form_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
		return 0;
	}
	return new rank(f,lower_is_better);
}


int parser::get_expression1(QString& s,int beg,formula*& f)
{
	int cur=get_expression2(s,beg,f);
	if(s.length()>cur+1)
	{
		if(s[cur]=='|' && s[cur+1]!='|')
		{
			formula* ff=f;
			f=new formula;
			f->id=f_or;
			f->op1=ff;
			cur=get_expression1(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,T->str[ins_2_arg_or]);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_expression2(QString& s,int beg,formula*& f)
{
	int cur=get_expression3(s,beg,f);
	if(s.length()>cur)
	{
		if(s[cur]=='&')
		{
			formula* ff=f;
			f=new formula;
			f->id=f_and;
			f->op1=ff;
			cur=get_expression2(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,T->str[ins_2_arg_and]);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_expression3(QString& s,int beg,formula*& f)
{
	int cur=beg;
	ignore_spaces(s,cur);
	if(s.length()>cur)
	{
		if(s[cur]=='[')
		{
			cur=get_expression1(s,++cur,f);
			if(sym(s,cur++)!=']') throw excparser(cur,T->str[ins_sqbracket_db]);
		}
		else
		{
			f=new formula;
			cur=get_formula1(s,cur,f->op1);
			if(sym(s,cur)=='<')
				f->id=f_less;
			else if(sym(s,cur)=='>')
				f->id=f_greater;
			else if(sym(s,cur)=='=')
				f->id=f_equal;
			else
				throw excparser(cur,T->str[ins_comparison]);
			cur=get_formula1(s,++cur,f->op2);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}

int parser::get_formula1(QString& s,int beg,formula*& f)
{
	int cur=get_formula2(s,beg,f);
	while(s.length()>cur)
	{
		if(sym(s,cur)=='+' || sym(s,cur)=='-')
		{
			formula* ff=f;
			QCharRef sign=s[cur];
			f=new formula;
			f->id=(s[cur]=='+')?f_plus:f_minus;
			f->op1=ff;
			cur=get_formula2(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,f->id==f_plus?T->str[ins_2_arg_plus]:T->str[ins_2_arg_minus]);
		}
		else break;
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_formula2(QString& s,int beg,formula*& f)
{
	int cur=get_formula3(s,beg,f);
	while(s.length()>cur)
	{
		if(s[cur]=='*' || s[cur]=='/')
		{
			formula* ff=f;
			QCharRef sign=s[cur];
			f=new formula;
			f->id=(s[cur]=='*')?f_times:f_div;
			f->op1=ff;
			cur=get_formula3(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,f->id==f_times?T->str[ins_2_arg_times]:T->str[ins_2_arg_div]);
		}
		else break;
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_formula3(QString& s,int beg,formula*& f)
{
	int cur=get_formula4(s,beg,f);
	if(s.length()>cur)
	{
		if(s[cur]=='^')
		{
			formula* ff=f;
			f=new formula;
			f->id=f_power;
			f->op1=ff;
			cur=get_formula3(s,++cur,f->op2);
			if(!f->op2) throw excparser(cur,T->str[ins_2_arg_pow]);
		}
	}
	ignore_spaces(s,cur);
	return cur;
}
int parser::get_formula4(QString& s,int beg,formula*& f)
{
	int cur=beg;
	ignore_spaces(s,cur);
	if(sym(s,cur)=='-')
	{
		cur=get_formula4(s,++cur,f);
		formula* ff=f;
		f=new formula(ff);
		f->id=f_unary_minus;
	}
	else if(sym(s,cur)=='(')
	{
		cur=get_formula1(s,++cur,f);
		if(sym(s,cur++)!=')') throw excparser(cur,T->str[ins_bracket_db]);
	}
	else if(sym(s,cur).isDigit())
	{
		f=new formula;
		f->id=f_none;
		f->value=get_number(s,cur);
	}
	// functions
	else
	{
		QString id=get_id(s,cur);
		f=new formula;
		if(id=="sqr")
		{
			get_function(s,cur,f,f_sqr);
		}
		else if(id=="exp")
		{
			get_function(s,cur,f,f_exp);
		}
		else if(id=="log")
		{
			get_function(s,cur,f,f_log);
		}
		else if(id=="rank")
		{
			f->id=f_rank;
			if(sym(s,cur++)=='(')
			{
				f->intvalue=(int)get_number(s,cur);
			}
			else	// we assume it's the time rank (aka rank #0)
			{
				f->intvalue=0;
				cur-=2;
			}
			++cur;
		}
		else if(!name_to_id.contains(id))
		{
			// constants
			for(int i=0;i<n_constants;++i)
				if(id==constants_id[i])
				{
					f->id=f_int;
					f->intvalue=constants[i];
					ignore_spaces(s,cur);
					return cur;
				}
			throw excparser(cur,T->str[ins_unknown_id]+" "+id);
		}
		else
			f->id=name_to_id.value(id);
	}
	ignore_spaces(s,cur);
	return cur;
}

int parser::get_special_cf(QString& s,int beg,c_formula*& f)
{
	int cur=beg;
	f=new c_formula;
	QString id=get_id(s,cur);
	if(!(f->type=name_to_type[id])) throw excparser(cur,T->str[ins_unknown_id]+" "+id);
	return cur;
}

int parser::get_cf(QString& s,int beg,c_formula*& f)
{
	int cur=beg;
	if(s[cur]=='!') return get_special_cf(s,++cur,f);
	f=new c_formula;
	cur=get_formula1(s,cur,f->f);
	if(s[cur]=='{')
	{
		f->precision=get_int(s,++cur);
		if(s[cur]=='}') ++cur;
	}
	ignore_spaces(s,cur);
	if(s.length()<=cur+3) return cur;
	if(s[cur]=='\'')
	{
		while(s.length()>++cur && s[cur]!='\'')	f->separator+=s[cur];
		++cur;
		cur=get_cf(s,cur,f->tail);
	}
	return cur;
}

void parser::find_pipe(QString& s,int& cur)
{
	while(cur<s.length() && s[cur]!=pipe) ++cur;
}
void parser::find_double_pipe(QString& s,int& cur)
{
	while(cur+1<s.length() && (s[cur]!=pipe || s[cur+1]!=pipe)) ++cur;
}

// counter ::= name[menu] | counterformula | b[<,>]<value> | r[<,>][nopopup] 
counter* parser::parse_counter(QString s)
{
	int cur=0;
	counter* c=0;
	c_formula* f=0;
	QString name,menu;
	int type=0;
	try
	{
		name=get_string_with_spaces(s,cur).trimmed();
		ignore_spaces(s,cur);
		if(s[cur]=='[') 
		{
			menu=get_string_with_spaces(s,++cur).trimmed();
			ignore_spaces(s,++cur);
		}
		else menu=name;
		if(s[cur]=='{') type=get_int(s,++cur);	
		find_pipe(s,cur);
		
		s=s.toLower();
		cur=get_cf(s,++cur,f);

		c=new counter();
		c->set_name(name);
		c->set_menu(menu);
		c->type=type;
		c->formula=f;
		
		find_pipe(s,cur);
		ignore_spaces(s,++cur);
		if(s[cur]=='b')
		{
			c->bold=true;
			c->bold_lower=(s[++cur]=='<');
			c->bold_value=get_number(s,++cur); 
			find_pipe(s,cur);ignore_spaces(s,++cur);
		}
		if(s[cur]=='r')
		{
			c->rec=new record;
			c->rec->lower_is_better=(s[++cur]=='<');
			if(s[++cur]=='n') c->rec->popup=false;
			find_pipe(s,cur);ignore_spaces(s,++cur);
		}
		if(s[cur]=='w')
		{
			++cur;
			c->row=get_int(s,cur);
			if(s[cur]==',') ++cur;
			c->column=get_int(s,cur);
		}
		
		cout<<"counter "<<c->print()<<endl;
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_cf_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;
		return 0;
	}
	return c;
}

autosave_condition* parser::parse_autosave_condition(QString s)
{
	int cur=0;
	int beg;
	autosave_condition* c=0;
	formula* f=0;
	QString name;
	try
	{
		name=get_string_with_spaces(s,cur).trimmed();
		find_pipe(s,cur);
		beg=++cur;
		find_double_pipe(s,cur);
		
		f=parse_expression(s.mid(beg,cur-beg));
		cur+=2;
				
		c=new autosave_condition;
		c->name=name;
		c->condition=f;
		beg=cur;
		find_pipe(s,cur);
		c->folder=s.mid(beg,cur-beg).trimmed();
		if(c->folder=="" || !QDir(c->folder).exists()) c->folder=video_dir+c->folder;
		if(!c->folder.size() || c->folder[c->folder.size()-1]!=slash) c->folder+=slash;
		if(!QDir(c->folder).exists())
		{
			err(T->str[ins_folder]+" "+c->folder+" "+T->str[ins_doesnt_exist]+". "
				+T->str[ins_condition]+" "+c->name+" "+T->str[ins_will_be_reset]);
			c->folder=video_dir;
		}
		find_pipe(s,cur);
		ignore_spaces(s,++cur);
		beg=cur;find_pipe(s,cur);
		c->filename=parse_string(s.mid(beg,cur-beg).trimmed());
		ignore_spaces(s,++cur);
		if(s.length()>cur && s[cur]=='o')
		{
			c->enabled=false;
			find_pipe(s,cur);ignore_spaces(s,++cur);
		}
		if(s.length()>cur && s[cur]=='a')
		{
			c->rawvf=false;
			find_pipe(s,cur);ignore_spaces(s,++cur);			
		}
		if(s.length()>cur && s[cur]=='t')
		{
			c->terminal=true;
			find_pipe(s,cur);ignore_spaces(s,++cur);
		}
		if(s.length()>cur && s[cur]=='p')
		{
			c->enabled_popup=true;
			beg=cur+=2;find_pipe(s,cur);
			c->popup_string=parse_string(s.mid(beg,cur-beg));
			ignore_spaces(s,++cur);
			if(s.length()>cur && s[cur]=='o')
			{
				c->enabled_popup=false;
				find_pipe(s,cur);ignore_spaces(s,++cur);
			}
		}

		cout<<"autosave "<<c->print()<<endl;
	}
	catch(excparser e)
	{
		err(s+"\n\n"+T->str[ins_as_err]+"("+e.msg+"); "+T->str[ins_was_noticed]+
			QString::number(e.sym));
		if(f) delete f;f=0;
		return 0;
	}
	return c;
}

