#include "ranktree.h"

rank_tree_node::rank_tree_node(double v):value(v),red(true),size(1),p(0),l(0),r(0)
{
}

rank_tree_node::rank_tree_node(double v,rank_tree_node* p):value(v),red(true),size(1),p(p),l(0),r(0)
{
}

rank_tree_node::~rank_tree_node()
{
	if(l) delete l;
	if(r) delete r;
}

QString rank_tree_node::print()
{
	QString s;
	if(l) s=l->print()+" ";
	s+=QString::number(value);
	if(r) s+=" "+r->print();
	return s;
}

inline int size(rank_tree_node* t)
{
	return t?t->size:0;
}
inline bool red(rank_tree_node* t)
{
	return t?t->red:false;
}

rank_tree_node* rank_tree::insert(double v, int& rank, bool lower_is_better)
{
	if(!root) return root=new rank_tree_node(v);
	
	rank_tree_node* x=root;
	rank_tree_node* y;
	bool to_right;
	do
	{	
		y=x;
		if(( to_right = ((x->value>v) ^ lower_is_better) ))
		{
			rank+=size(x->l)+1;
			++x->size;
			x=x->r;
		}
		else
		{
			++x->size;
			x=x->l;
		}
	}
	while(x);
	
	x=new rank_tree_node(v,y);
	if(to_right) y->r=x; else y->l=x;

	return x;
}

void rank_tree::left_rotate(rank_tree_node* t)
{
	rank_tree_node* y=t->r;
	t->r=y->l;
	if(y->l) y->l->p=t;
	y->p=t->p;
	if(t->p)
		if(t->p->l==t)
			t->p->l=y;
		else
			t->p->r=y;
	else
		root=y;
	y->l=t;
	t->p=y;
	y->size=t->size;
	t->size=size(t->l)+size(t->r)+1;
}
void rank_tree::right_rotate(rank_tree_node* t)
{
	rank_tree_node* y=t->l;
	t->l=y->r;
	if(y->r) y->r->p=t;
	y->p=t->p;
	if(t->p)
		if(t->p->l==t)
			t->p->l=y;
		else
			t->p->r=y;
	else
		root=y;
	y->r=t;
	t->p=y;
	y->size=t->size;
	t->size=size(t->l)+size(t->r)+1;
}

int rank_tree::insert(double v,bool lower_is_better)
{
	int rank=1;
	rank_tree_node* x=insert(v,rank,lower_is_better);
	while(x!=root && x->p->red)
	{
		if(x->p==x->p->p->l)
		{
			rank_tree_node* y=x->p->p->r;
			if(red(y))
			{
				x->p->red=false;
				y->red=false;
				x->p->p->red=true;
				x=x->p->p;
			}
			else
			{
				if(x->p->r==x)
				{
					x=x->p;
					left_rotate(x);
				}
				x->p->red=false;
				x->p->p->red=true;
				right_rotate(x->p->p);
			}
		}
		else
		{
			rank_tree_node* y=x->p->p->l;
			if(red(y))
			{
				x->p->red=false;
				y->red=false;
				x->p->p->red=true;
				x=x->p->p;
			}
			else
			{
				if(x->p->l==x)
				{
					x=x->p;
					right_rotate(x);
				}
				x->p->red=false;
				x->p->p->red=true;
				left_rotate(x->p->p);
			}
		}
	}
	
	root->red=false;
	
	return rank;
}


rank_tree::rank_tree():root(0)
{
}

rank_tree::~rank_tree()
{
	if(root) delete root;
}

QString rank_tree::print()
{
	return root ? root->print() : "";
}

