#include "settings.h"
#include "settings-names.h"
#include "settings-names-skin.h"

settings::settings(const QString& filename)
{
	store.setFileName(filename);
}

bool settings::read(const QString& err_msg)
{
	QString line,name,value;
	if(store.open(QIODevice::ReadOnly))
	{
		while(!store.atEnd())
		{
			int sp;
			line=QString::fromUtf8(store.readLine());
			// if it's not a valid entry ("Name = Value"), ignore
			if( (sp=line.indexOf('=')) == -1) continue;	
			name=line.left(sp).trimmed();
			value=line.mid(sp+1).trimmed();
			data[name]=value;
		}
		store.close();
		return true;
	}
	else
	{
		err(err_msg);
		return false;
	}
}

void settings::defaultsettings()
{
	set(sMainWindowX,"100");
	set(sMainWindowY,"100");
	set(sBoardWidth,"8");
	set(sBoardHeight,"8");
	set(sBoardMines,"10");
	set(sCustomWidth,30);
	set(sCustomHeight,24);
	set(sCustomMines,200);
	set(sDateTemplate,"YYYY-MM-DD HH:MM:SS");
	set(sVideoDir,normal_settings_dir);
	set(sBoardDir,normal_settings_dir);
	set(sIRCCopyLast,"1");
	set(sIRCCopyBlast,"1");
	set(sShowCountersWindow,"1");
	set(sCountersAttached,"1");
	set(sCountersSeparated,"1");
	set(sFlagMinesAfterWin,"1");
	set(sTimeTrial+sLevel+"1","15");
	set(sTimeTrial+sLevel+"2","6");
	set(sTimeTrial+sLevel+"3","2");
	set(sBeepBeg,"10000");
	set(sBeepInt,"40000");
	set(sBeepExp,"100000");
	set(sRestartBegLimit,"10000");
	set(sRestartIntLimit,"40000");
	set(sRestartExpLimit,"100000");
	set(sCloneHesitationTime,"1000");
	set(sLuckIterations,"1000000");
	set(sProbabilitiesIterations,"1000000");
	set(sGenerationIterations,"100000");
	set(sMarathonLength+sLevel+"1","16");
	set(sMarathonLength+sLevel+"2","4");
	set(sMarathonLength+sLevel+"3","2");
	set(sRecordsOnAHSWindow,"5");
	set(sWheelAsChord,"1");
}

void settings::defaultskin()
{
	set(sName,"Default");
	set(sCoeff,"1");
	set(sSquareSize,"16");
	set(sHeadSize,"46");
	set(sSmileyWidth,"27");
	set(sSmileyHeight,"27");
	set(sDisplayWidth,"13");
	set(sDisplaySmallWidth,"9");
	set(sDisplayHeight,"23");
	set(sBackground+sRed,"192");
	set(sBackground+sGreen,"192");
	set(sBackground+sBlue,"192");
	set(sLabelBackground+sRed,"224");
	set(sLabelBackground+sGreen,"224");
	set(sLabelBackground+sBlue,"224");
	set(sLabelFont+sRed,"0");
	set(sLabelFont+sGreen,"0");
	set(sLabelFont+sBlue,"0");
	set(sDisplayFrame1+sRed,"0");
	set(sDisplayFrame1+sGreen,"0");
	set(sDisplayFrame1+sBlue,"0");
	set(sDisplayFrame2+sRed,"255");
	set(sDisplayFrame2+sGreen,"255");
	set(sDisplayFrame2+sBlue,"255");
	
	set(sCriticalWidth,"8");
	set(sCriticalForSmileyWidth,"8");
	set(sMaxWidth,"50");
	set(sMaxHeight,"30");
	
	set(sLabelVerticalOffset,"4");
	set(sLabelHorizontalOffset,"4");
	set(sLabelTextVerticalOffset,"3");
	set(sLabelHeight,"16");
	set(sRedMinesHorizontalOffset,"6");
	set(sRedTimerHorizontalOffset,"-5");
	set(sSmileySmallBoardHorizontalOffset,"-4");
	set(sSmileyVerticalOffset,"4");
}

QString settings::get(const QString& name)
{
	return data[name];
}

int settings::getint(const QString& name)
{
	return data[name].toInt();
}

double settings::getdouble(const QString& name)
{
	return data[name].toDouble();
}

void settings::set(const QString& name,const QString& value)
{
	data[name]=value;
}
void settings::set(const QString& name,int value)
{
	data[name]=QString::number(value);
}
void settings::set(const QString& name,double value)
{
	data[name]=QString::number(value);
}

bool settings::write(const QString& err_msg)
{
	if(store.open(QIODevice::WriteOnly | QIODevice::Truncate))
	{
		for(QMap<QString,QString>::iterator i=data.begin();i!=data.end();++i)
		{
			store.write((i.key() + " = " + i.value() + "\n").toUtf8());
		}
		store.close();
		return true;
	}
	else
	{
		err(err_msg);
		return false;
	}
}

