#include "skin.h"
#include "settings-names-skin.h"
#include "skin-default.h"

skin::skin(translation* T):loaded(false),T(T)
{
}

QColor get_color(settings& s,const QString& name)
{
	int r=s.getint(name+sRed);
	int g=s.getint(name+sGreen);
	int b=s.getint(name+sBlue);
	return QColor(r,g,b);
}

void skin::load(const QString& desc)
{
	settings s(desc);
	QString s_dir;
  	QString old_cur=QDir::currentPath();
  	
	loaded=false;
	if(desc=="") return;
	
	s.defaultskin();	// to prevent troubles caused by a corrupted skin file
	loaded=s.read(T->str[ins_e_open_skin]);
	if(!loaded) return;
	
	skin_path=desc;
	s_dir=s.get(sSkinDir);
	QDir::setCurrent(settings_dir+s_dir);
	
	coeff=s.getdouble(sCoeff);

	square_size=s.getint(sSquareSize);
	head_size=s.getint(sHeadSize);
	topborder_size=s.getint(sTopBorderSize);
	smiley_width=s.getint(sSmileyWidth);
	smiley_height=s.getint(sSmileyHeight);
	display_width=s.getint(sDisplayWidth);
	small_display_width=s.getint(sDisplaySmallWidth);
	display_height=s.getint(sDisplayHeight);
	background=get_color(s,sBackground);
	label_background=get_color(s,sLabelBackground);
	label_font=get_color(s,sLabelFont);
	display_frame1=get_color(s,sDisplayFrame1);
	display_frame2=get_color(s,sDisplayFrame2);

	rm_h_offset=s.getint(sRedMinesHorizontalOffset);
	rt_h_offset=s.getint(sRedTimerHorizontalOffset);
	smiley_h_o_offset=s.getint(sSmileySmallBoardHorizontalOffset);
	smiley_v_offset=s.getint(sSmileyVerticalOffset);
	
	crit_width=s.getint(sCriticalWidth);
	crit_width_2=s.getint(sCriticalForSmileyWidth);
	max_width=s.getint(sMaxWidth);
	max_height=s.getint(sMaxHeight);

	arrow=get_color(s,sArrowColor);
	arrow_border=get_color(s,sArrowBorderColor);
	arrow_opacity=s.getdouble(sArrowOpacity);
	for(int i=0;i<4;++i) mouse_path[i]=get_color(s,sMousePathColor[i]);
	opening_border=get_color(s,sOpeningBorder);
	mouse_path_rect_size=s.getint(sMousePathRectSize);
	opening_border_thickness=s.getint(sOpeningBorderThickness);
	
	label_h_offset=s.getint(sLabelHorizontalOffset);
	label_v_offset=s.getint(sLabelVerticalOffset);
	label_text_offset=s.getint(sLabelTextVerticalOffset);
	label_height=s.getint(sLabelHeight);
		
	counter_width=s.getint(sCounterWidth);
	counter_height=s.getint(sCounterHeight);
	counter_font_size=s.getint(sCounterFontSize);
	counter_font.setStyleHint(QFont::SansSerif);
	counter_font.setPointSize(counter_font_size);
	counter_name_back=get_color(s,sCounterNameBackground);
	counter_value_back=get_color(s,sCounterValueBackground);
	
	chartEWidth=s.getdouble(sChartElementaryWidth);	
	chartHeight=s.getint(sChartHeight);
	chartLength=s.getint(sChartLength);
	for(int i=0;i<=1;++i) 
	{
		chartLabelVOffset[i]=s.getint(sChartLabelVOffset+QString::number(i+1));
		chartLabelHOffset[i]=s.getint(sChartLabelHOffset+QString::number(i+1));
	}
	chartLineThickness=s.getint(sChartLineThickness);
	chartLineThicknessSpec=s.getint(sChartThickLineThickness);
	chartBackground=get_color(s,sChartBackground);
	chartSecondFrame=get_color(s,sChartSecondFrame);
	chartSecondLabel=get_color(s,sChartSecondLabel);
	chartSecondLabelVOffset=s.getint(sChartSecondLabelVOffset);
	chartLeft=get_color(s,sChartLeft);
	chartRight=get_color(s,sChartRight);
	chartMiddle=get_color(s,sChartMiddle);
	chartOpening=get_color(s,sChartOpening);
	chartIsland=get_color(s,sChartIsland);
	chartLeftH=s.getint(sChartLeft+sHeight_);
	chartRightH=s.getint(sChartRight+sHeight_);
	chartMiddleH=s.getint(sChartMiddle+sHeight_);
	chartOpeningH=s.getint(sChartOpening+sHeight_);
	chartIslandH=s.getint(sChartIsland+sHeight_);
	
	sq.load(s.get(sSquare));
	sq_pr.load(s.get(sSquarePressed));
	question.load(s.get(sQuestion));
	question_pr.load(s.get(sQuestionPressed));
	for(int i=0;i<10;++i) border[i].load(s.get(sBorder+QString::number(i)));
	for(int i=0;i<9;++i) number[i].load(s.get(sNumber+QString::number(i)));
	for(int i=0;i<9;++i) numberpop[i].load(s.get(sNumberPopped+QString::number(i)));
	flag.load(s.get(sFlag));
	mine.load(s.get(sMine));
	minepop.load(s.get(sMinePopped));
	blasted_mine.load(s.get(sBlast));
	flag_false.load(s.get(sFlagFalse));
	flag_wasted.load(s.get(sFlagWasted));
	for(int i=0;i<10;++i) display[i].load(s.get(sDisplay+QString::number(i)));
	for(int i=0;i<10;++i) smalldisplay[i].load(s.get(sDisplaySmall+QString::number(i)));
	display_minus.load(s.get(sDisplayMinus));
	display_off.load(s.get(sDisplayOff));
	smalldisplay_off.load(s.get(sSmallDisplayOff));
	lborder.load(s.get(sLeftBorder));
	rborder.load(s.get(sRightBorder));
	tborder.load(s.get(sTopBorder));
	for(int i=0;i<n_modes;++i)
	{
		smiley[i].load(s.get(sSmiley+QString::number(i)));
		smiley_stress[i].load(s.get(sSmileyStress+QString::number(i)));
		smiley_click[i].load(s.get(sSmileyPressed+QString::number(i)));
		smiley_win[i].load(s.get(sSmileyWin+QString::number(i)));
		smiley_fail[i].load(s.get(sSmileyFail+QString::number(i)));
	}
	
	QPixmap p;
	p.load(s.get(sVideoButtonPlay));
	b_play=QIcon(p);
	p.load(s.get(sVideoButtonPause));
	b_pause=QIcon(p);
	p.load(s.get(sVideoButtonStop));
	b_stop=QIcon(p);
	
	p.load(s.get(sIcon));
	program_icon=QIcon(p);
	
	name=s.get(sName);

	QDir::setCurrent(old_cur);
}

void skin::load_default()
{
	QByteArray b(default_skin,sizeof(default_skin)/sizeof(char));
	QDataStream d(&b,QIODevice::ReadOnly);
	load_from_ds(d);
}

void skin::load_from_ds(QDataStream& d)
{
	d>>coeff>>square_size>>head_size>>topborder_size>>display_width>>small_display_width>>display_height
	>>smiley_width>>smiley_height>>smiley_v_offset>>label_h_offset>>label_v_offset>>label_height
	>>label_text_offset>>crit_width>>crit_width_2>>max_width>>max_height>>background>>label_background>>
	label_font>>display_frame1>>display_frame2>>rm_h_offset>>rt_h_offset>>smiley_h_o_offset>>
	arrow>>arrow_border>>arrow_opacity;
	for(int i=0;i<4;++i) d>>mouse_path[i];
	d>>opening_border>>mouse_path_rect_size>>opening_border_thickness>>counter_width>>counter_height>>
	counter_font_size>>counter_font>>counter_name_back>>counter_value_back>>chartEWidth>>chartLength>>
	chartHeight>>chartLabelVOffset[0]>>chartLabelVOffset[1]>>chartLabelHOffset[0]>>chartLabelHOffset[1]>>
	chartLineThickness>>chartLineThicknessSpec>>chartBackground>>chartSecondFrame>>chartSecondLabel>>
	chartSecondLabelVOffset>>chartLeft>>chartRight>>chartMiddle>>chartOpening>>chartIsland>>
	chartLeftH>>chartRightH>>chartMiddleH>>chartOpeningH>>chartIslandH>>name>>
	sq>>sq_pr>>sq_empty>>question>>question_pr;
	for(int i=0;i<10;++i) d>>number[i];
	for(int i=0;i<10;++i) d>>numberpop[i];
	for(int i=0;i<10;++i) d>>display[i];
	for(int i=0;i<10;++i) d>>smalldisplay[i];
	d>>display_minus>>display_off>>smalldisplay_off;
	for(int i=0;i<10;++i) d>>border[i];
	d>>blasted_mine>>mine>>minepop>>flag>>flag_false>>flag_wasted>>lborder>>rborder>>tborder;
	for(int i=0;i<n_modes;++i) d>>smiley[i]>>smiley_click[i]>>smiley_stress[i]>>
		smiley_win[i]>>smiley_fail[i];
	d>>b_play>>b_pause>>b_stop>>program_icon;
}

void skin::load_raw(const QString& desc)
{
	QFile f(desc);
	QDataStream d(&f);
	
	if(f.open(QIODevice::ReadOnly)) load_from_ds(d);
}
void skin::write_raw(const QString& desc)
{
	QFile f(desc);
	QDataStream d(&f);
	
	if(f.open(QIODevice::WriteOnly | QIODevice::Truncate))
	{
		d<<coeff<<square_size<<head_size<<topborder_size<<display_width<<small_display_width<<display_height
		<<smiley_width<<smiley_height<<smiley_v_offset<<label_h_offset<<label_v_offset<<label_height
		<<label_text_offset<<crit_width<<crit_width_2<<max_width<<max_height<<background<<label_background<<
		label_font<<display_frame1<<display_frame2<<rm_h_offset<<rt_h_offset<<smiley_h_o_offset<<
		arrow<<arrow_border<<arrow_opacity;
		for(int i=0;i<4;++i) d<<mouse_path[i];
		d<<opening_border<<mouse_path_rect_size<<opening_border_thickness<<counter_width<<counter_height<<
		counter_font_size<<counter_font<<counter_name_back<<counter_value_back<<chartEWidth<<chartLength<<
		chartHeight<<chartLabelVOffset[0]<<chartLabelVOffset[1]<<chartLabelHOffset[0]<<chartLabelHOffset[1]<<
		chartLineThickness<<chartLineThicknessSpec<<chartBackground<<chartSecondFrame<<chartSecondLabel<<
		chartSecondLabelVOffset<<chartLeft<<chartRight<<chartMiddle<<chartOpening<<chartIsland<<
		chartLeftH<<chartRightH<<chartMiddleH<<chartOpeningH<<chartIslandH<<name<<
		sq<<sq_pr<<sq_empty<<question<<question_pr;
		for(int i=0;i<10;++i) d<<number[i];
		for(int i=0;i<10;++i) d<<numberpop[i];
		for(int i=0;i<10;++i) d<<display[i];
		for(int i=0;i<10;++i) d<<smalldisplay[i];
		d<<display_minus<<display_off<<smalldisplay_off;
		for(int i=0;i<10;++i) d<<border[i];
		d<<blasted_mine<<mine<<minepop<<flag<<flag_false<<flag_wasted<<lborder<<rborder<<tborder;
		for(int i=0;i<n_modes;++i) d<<smiley[i]<<smiley_click[i]<<smiley_stress[i]<<
			smiley_win[i]<<smiley_fail[i];
		d<<b_play<<b_pause<<b_stop<<program_icon;
	}
}



