#include "timer.h"

const int interval=30;

timer::timer():coeff(1)
{
	connect(&t,SIGNAL(timeout()),this,SLOT(send_signal()));
	t.setInterval(interval);
	t.setSingleShot(false);
}

void timer::start(int _coeff)
{
	time.start();
	t.start();
	emit tick(last_stop=add=0);
	emit tick_since_stop(0);
	set_coeff(_coeff);
}

void timer::stop()
{
	if(t.isActive())
	{
		t.stop();
		send_signal();
	}
}
void timer::stop(int end_time)
{
	if(t.isActive())
	{
		t.stop();
		emit tick(end_time);
		emit tick_since_stop(end_time-last_stop);
		add=end_time;
	}
}

void timer::restart()
{
	coeff=1.0;
	last_stop=add;
	time.start();
	if(!t.isActive()) t.start();
}

int timer::send_signal()
{
	int r = time.restart();
	int ticked=(int)(r*coeff);
	if(ticked>0) add += ticked; else ++add;
	emit tick(add);
	emit tick_since_stop(add-last_stop);
	return add;
}

int timer::get_time()
{
	return paused() ? add : add+time.elapsed();
}

int timer::get_time_since_stop()
{
	return paused() ? add-last_stop : add+time.elapsed()-last_stop;
}

double timer::get_coeff()
{
	return coeff;
}

void timer::pause()
{
	if(paused()) return;
	send_signal();
	t.stop();
}

void timer::resume()
{
	time.start();
	t.start();
}

void timer::set_time(int _time)
{
	add=_time;
	time.start();
	emit tick(_time);
	emit tick_since_stop(_time);
}

void timer::set_coeff(int _coeff)
{
	coeff=_coeff*0.01;
}

bool timer::paused()
{
	return !t.isActive();
}

