#include "videocontrol.h"

videocontrol::videocontrol(translation* T,skin* sk,int video_duration,bool enable_chart,clickevents* ce):
chart(ce,sk),T(T),sk(sk),video_duration(video_duration)
{
	layout=new QGridLayout(this);

	layout->addWidget(&time,0,0,1,5);
	layout->addWidget(&play,1,0,1,1);
	layout->addWidget(&pause,1,1,1,1);
	layout->addWidget(&stop,1,2,1,1);
	layout->addWidget(&lTime,1,3,1,1);
	layout->addWidget(&speed,1,4,1,1);
	speed.setContextMenuPolicy(Qt::CustomContextMenu);
	
	layout->addWidget(&mousePath,2,0,1,3);
	layout->addWidget(&showOpenings,2,3,1,1);
	layout->addWidget(&revealSquares,2,4,1,1);
	if(enable_chart)
	{
		layout->addWidget(&events,3,0,1,5);
		layout->addWidget(&chart,4,0,1,5);
	}
	
	row=0;
	
	time.setOrientation(Qt::Horizontal);
	time.setTickInterval(1000);
	time.setTickPosition(QSlider::TicksBelow);
	time.setPageStep(1000);
	time.setRange(0,video_duration);
	time.setSliderPosition(0);
	speed.setOrientation(Qt::Horizontal);
	speed.setTickInterval(100);
	speed.setTickPosition(QSlider::TicksBelow);
	speed.setRange(0,500);
	speed.setSliderPosition(100);

	play.setIcon(sk->b_play);
	pause.setIcon(sk->b_pause);
	stop.setIcon(sk->b_stop);

	mousePath.setText(T->str[ins_mouse_path]);
	revealSquares.setText(T->str[ins_reveal_squares]);
	showOpenings.setText(T->str[ins_ops]);

	events.insertColumn(0);
	events.insertColumn(1);
	events.setHorizontalHeaderItem(0,new QTableWidgetItem(T->str[ins_time]));
	events.setHorizontalHeaderItem(1,new QTableWidgetItem(T->str[ins_event]));
	
	connect(&time,SIGNAL(sliderPressed()),this,SLOT(time_click()));
	connect(&time,SIGNAL(sliderMoved(int)),this,SLOT(time_move(int)));
	connect(&time,SIGNAL(sliderReleased()),this,SLOT(time_change()));
	connect(&speed,SIGNAL(sliderMoved(int)),this,SLOT(speed_move(int)));
	connect(&speed,SIGNAL(customContextMenuRequested(const QPoint&)),this,SLOT(speed_right_click(const QPoint&)));
	connect(&play,SIGNAL(clicked()),this,SIGNAL(play_video()));
	connect(&pause,SIGNAL(clicked()),this,SIGNAL(pause_video()));
	connect(&stop,SIGNAL(clicked()),this,SIGNAL(stop_video()));

	connect(&mousePath,SIGNAL(toggled(bool)),this,SIGNAL(toggle_path(bool)));
	connect(&revealSquares,SIGNAL(toggled(bool)),this,SIGNAL(toggle_reveal(bool)));
	connect(&showOpenings,SIGNAL(toggled(bool)),this,SIGNAL(toggle_openings(bool)));
	
	connect(&events,SIGNAL(cellDoubleClicked(int,int)),this,SLOT(goto_event(int,int)));
	
	moving=false;
	l_e_time='/'+QString::number(video_duration/1000.0,'f',3);
	tick(0);
	
	Qt::WindowFlags f=windowFlags();
	f&=~(Qt::WindowMinMaxButtonsHint);
	f|=Qt::Drawer;
	setWindowFlags(f);
	setWindowTitle(T->str[ins_video_control]);
	setWindowIcon(sk->program_icon);
}

void videocontrol::add_event(int time,int id)
{
	int names[]={0,ins_Blast,ins_wasted_click,ins_guessed_click,ins_misflag,ins_wasted_flag,
		ins_guessed_flag,ins_unflag,ins_misunflag,ins_wasted_chord,ins_wasted_chord_15,ins_hesitation,
		ins_rilian_click,ins_question};
	QTableWidgetItem* l_t=new QTableWidgetItem();
	QTableWidgetItem* l_n=new QTableWidgetItem();

	l_t->setText(QString::number(time/1000.0));
	l_n->setText(T->str[names[id]]);
	l_t->setFlags(l_t->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
	l_n->setFlags(l_n->flags() & (~Qt::ItemIsSelectable & ~Qt::ItemIsEditable));
	events.insertRow(row);
	events.setItem(row,0,l_t);
	events.setItem(row++,1,l_n);

	events.resizeColumnsToContents();	
}

void videocontrol::time_click()
{
	moving=true;
}

void videocontrol::time_move(int pos)
{
	if(pos>video_duration) pos=video_duration;
	chart.time_move(cur=pos);
	lTime.setText(QString::number(pos/1000.0,'f',3)+l_e_time);
}

void videocontrol::time_change()
{
	moving=false;
	emit time_changed(time.sliderPosition());
	chart.repaint();
}

void videocontrol::speed_move(int pos)
{
	if(pos==0) speed.setSliderPosition(1);
	emit speed_changed(speed.sliderPosition());
}
void videocontrol::set_normal_speed()
{
	speed.setSliderPosition(100);
	emit speed_changed(speed.sliderPosition());
}
void videocontrol::speed_right_click(const QPoint&)
{
	set_normal_speed();
}

void videocontrol::tick(int tm)
{
	if(!moving)
	{
		if(tm>video_duration) tm=video_duration;
		chart.time_move(cur=tm);
		time.setSliderPosition(tm);
		lTime.setText(QString::number(tm/1000.0,'f',3)+l_e_time);
	}
}

int videocontrol::get_speed()
{
	return speed.sliderPosition();
}

void videocontrol::goto_event(int row, int column)
{
	int tm=(int)(events.item(row,0)->text().toDouble()*1000) - goto_offset + column;
	if(tm<0) tm=0;
	time.setSliderPosition(tm);
	emit time_changed(tm);
	chart.repaint();
}

void videocontrol::closeEvent(QCloseEvent *)
{
	emit exit_video();
}

