#include "cloud.h"

cloud::cloud():min_x(0),min_y(0),step_x(1),step_y(1),prec(3)
{
}

void cloud::reset()
{
	data.clear();
}

void cloud::add_point(double x,double y)
{
	if(!data.size() || x+1>max_x) max_x=x+1;
	if(x<min_x) min_x=x;
	if(!data.size() || y+1>max_y) max_y=y+1;
	if(y<min_y) min_y=y;
	data << QPair<double,double>(x,y);
}
double gap_to_step(double gap)
{
	if(gap>75)
		return 10.0;
	else if(gap>15)
		return 5.0;
	else if(gap>7.5)
		return 1.0;
	else if(gap>1.5)
		return 0.5;
	else if(gap>0.75)
		return 0.1;
	else if(gap>0.15)
		return 0.05;
	else
		return 0.01;	
}
void cloud::choose_step_x()
{
	step_x=gap_to_step(max_x-min_x);
}
void cloud::choose_step_y()
{
	step_y=gap_to_step(max_y-min_y);
}

void cloud::drawpoint(QPainter& p, int x, int y, double oldness)
{
	const int size=3;
	const int size2=size/2;
	const int size3=size2+1;
	double newness=1-oldness;
	QColor cur((int)(color_old.red()*newness+color_new.red()*oldness),
				(int)(color_old.green()*newness+color_new.green()*oldness),
				(int)(color_old.blue()*newness+color_new.blue()*oldness));
	QColor cur_b((int)(color_border_old.red()*newness+color_border_new.red()*oldness),
				(int)(color_border_old.green()*newness+color_border_new.green()*oldness),
				(int)(color_border_old.blue()*newness+color_border_new.blue()*oldness));
	p.setPen(QColor(cur_b));
	p.drawLine(x-size2,y-size3,x+size2,y-size3);
	p.drawLine(x-size2,y+size3,x+size2,y+size3);
	p.drawLine(x-size3,y-size2,x-size3,y+size2);
	p.drawLine(x+size3,y-size2,x+size3,y+size2);
	p.fillRect(x-size2,y-size2,size,size,QBrush(cur));
}

void cloud::draw_gray(QPainter& p,const int frame_size)
{	
	QColor gray(192,192,192);
	QBrush g(gray);
	p.fillRect(0,0,frame_size/2,height(),g);
	p.fillRect(width()-frame_size/2,0,frame_size/2,height(),g);
	p.fillRect(frame_size/2,0,width()-frame_size/2,frame_size/3,g);
	p.fillRect(frame_size/2,height()-frame_size/2,width()-frame_size,frame_size/2,g);
}

void cloud::paintEvent(QPaintEvent*)
{
	const int letter_width=3;
	const int letter_height_up=4;
	const int letter_height_down=9;

	const int frame_size=50+(prec-1)*letter_width*2;
	const int w=width()-frame_size;
	const int h=height()-frame_size;
	const int beg_w=frame_size/2;
	const int beg_w_up=beg_w+frame_size/10;
	const int beg_h=frame_size/3;
	const int end_h=h+frame_size/2-frame_size/3;
	const int end_h_n=height()-frame_size/2;
	const int end_h_up=end_h_n-frame_size/10;
	const QColor background(255,255,255);
	const QColor axis(0,0,0);
	const QColor axis_long(200,200,200);
	const QColor text(0,0,0);
	QFont f;
	
	f.setStyleHint(QFont::SansSerif);
	f.setPointSize(7);
	
	QPainter p;
	p.begin(this);
	p.fillRect(beg_w,beg_h,w,end_h,QBrush(background));
	p.setFont(f);
	if(min_x>=max_x || min_y>=max_y)
	{
		draw_gray(p,frame_size);
		return;
	}
	
	for(double d=min_x+step_x;d<max_x;d+=step_x)
	{
		int x=(int)(beg_w+(d-min_x)*w/(max_x-min_x));		
		
		p.setPen(axis);
		p.drawLine(x,end_h_n,x,end_h_up);
		p.setPen(axis_long); 
		p.drawLine(x,end_h_up,x,beg_h);
	}	
	for(double d=min_y+step_y;d<max_y;d+=step_y)
	{
		int y=(int)(end_h_n-(d-min_y)*h/(max_y-min_y));		
		
		p.setPen(axis);
		p.drawLine(beg_w,y,beg_w_up,y);
		p.setPen(axis_long); 
		p.drawLine(beg_w_up,y,w,y);
	}

	if(max_x-min_x==0) max_x=min_x+1;
	if(max_y-min_y==0) max_y=min_y+1;
	for(int i=0;i<data.size();++i)
	{
		int x=(int)(beg_w+(data[i].first-min_x)*w/(max_x-min_x));
		int y=(int)(end_h-(data[i].second-min_y)*h/(max_y-min_y));
		drawpoint(p,x,y,(double)i/data.size());
	}
	
	draw_gray(p,frame_size);
	
	for(double d=min_x+step_x;d<max_x;d+=step_x)
	{
		int x=(int)(beg_w+(d-min_x)*w/(max_x-min_x));		
		QString s=QString::number(d, 'f', prec);

		p.setPen(text);
		p.drawText(x-s.size()*letter_width, end_h+letter_height_down, s);
	}
	for(double d=min_y+step_y;d<max_y;d+=step_y)
	{
		int y=(int)(end_h-(d-min_y)*h/(max_y-min_y));		
		QString s=QString::number(d, 'f', prec);

		p.setPen(text);
		p.drawText(beg_w-s.size()*letter_width*2, y+letter_height_up, s);
	}

	p.setPen(axis);
	p.drawLine(beg_w,end_h,w,end_h);
	p.drawLine(beg_w,beg_h,beg_w,end_h);
	
	p.setPen(text);
	p.drawText(letter_width,beg_h-letter_height_up*2,y_axis);
	p.drawText(width()-letter_width*2*x_axis.size(), end_h+letter_height_down*2, x_axis);
	
	p.end();
}

