#include "histogram.h"

histogram::histogram():has_min_x(false),has_max_x(false),step_x(1.0),strings_on_x(false),prec(2)
{
}
histogram::~histogram()
{
}

void histogram::reset()
{
	has_min_x=has_max_x=false;
	data.clear();
	map.clear();
}

void histogram::add_result(double d)
{
	data << d;
	if(d<min_x || !has_min_x) min_x=(int)d;
	if(d+1>max_x || !has_max_x) max_x=(int)(d+1);
	has_min_x=has_max_x=true;
}

void histogram::fill_map()
{
	double b=has_min_x?min_x:0;
	double e=has_max_x?max_x:0;
	double nb=(e-b)/step_x;
	int inb=(int)nb;
	num_bars=(inb==nb)?inb:inb+1;

	map.clear();
	for(int i=0;i<data.size();++i)
	{
		++map[(int)((data[i]-b)/step_x)];
	}

	min_y=max_y=map[0];
	for(int i=1;i<=num_bars;++i)
		if(map[i]>max_y)
			max_y=map[i];
		else if(map[i]<min_y)
			min_y=map[i];
}

void histogram::paintEvent(QPaintEvent*)
{
	const int frame_size=50;
	const QColor background(255,255,255);
	const QColor frame(192,192,192);
	const double l10=log(10);
	const int letter_width=4;
	const int letter_height_up=5;
	const int letter_height_down=9;
	const int beg_w=frame_size/2;
	const int end_w=width()-frame_size/2;
	const int beg_h=frame_size/3;
	const int end_h=height()-frame_size/2;
	const int add_w=2;
	const int add_w2=2*add_w;
	QFont f;
	int w=width()-frame_size;
	int h=height()-frame_size;
	if(!num_bars) return;
	int bar_width=w/num_bars;
	double elem_h=(double)h/(max_y+1-min_y);
	
	if(elem_h<=0) elem_h=1;
	if(bar_width<=0) bar_width=1;
	
	f.setStyleHint(QFont::SansSerif);
	f.setPointSize(7);
	
	QPainter p;
	p.begin(this);
	p.fillRect(beg_w,beg_h,w,h+beg_w-beg_h,QBrush(background));
	p.fillRect(0,0,frame_size/2,height(),QBrush(frame));
	p.fillRect(end_w,0,frame_size/2,height(),QBrush(frame));
	p.fillRect(beg_w,0,width()-frame_size,beg_h,QBrush(frame));
	p.fillRect(beg_w,end_h,width()-frame_size,frame_size/2,QBrush(frame));
	p.setFont(f);
	if(!map.size() || min_x>=max_x || min_y>=max_y) return;
	
	double d=has_min_x?min_x:0;
	int j=0;
	for(int i=0;i<num_bars;++i)
	{
		int bar_h=(int)(elem_h*map[i]);
		int l=(int)(log(map[i])/l10);
		p.fillRect(beg_w+bar_width*i+add_w, end_h-bar_h, bar_width-add_w2, bar_h, QBrush(color));
		p.setPen(color_border);
		p.drawLine(beg_w+bar_width*i+add_w, end_h-bar_h, beg_w+bar_width*(i+1)-add_w, end_h-bar_h);
		p.drawLine(beg_w+bar_width*i+add_w, end_h-bar_h, beg_w+bar_width*i+add_w, end_h);
		p.drawLine(beg_w+bar_width*(i+1)-add_w, end_h-bar_h, beg_w+bar_width*(i+1)-add_w, end_h);
		
		p.setPen(QColor(0,0,0));
		if(map[i]==max_y || l*letter_width*2<=bar_width)
			p.drawText(beg_w+bar_width*i+bar_width/2+1-l*letter_width, end_h-letter_height_up-bar_h,
					QString::number(map[i]));
		if(strings_on_x)
			p.drawText(beg_w+bar_width*i, end_h+letter_height_down, strings.size()>i?strings[i]:"");
		else
		{
			int ll=(int)(log(d)/l10)+3;	// number of symbols
			++j;
			if(j*bar_width>ll*letter_width*2 || (!i && show_first) || (i==1 && !show_first))
			{
				j=0;
				p.drawText(beg_w+bar_width*i, end_h+letter_height_down, 
					QString::number(d, 'f', prec));
			}
			d+=step_x;
		}
	}
	if(strings_on_x)
		p.drawText(beg_w+bar_width*num_bars, end_h+letter_height_down,
					strings.size()>num_bars?strings[num_bars]:"");
	else
	{
		int ll=(int)(log(d)/l10)+3;	// number of symbols
		++j;
		if(j*bar_width>ll*letter_width*2)
		{
			p.drawText(beg_w+bar_width*num_bars, end_h+letter_height_down, 
				QString::number(d, 'f', prec));
		}
	}
	p.setPen(QColor(0,0,0));
	p.drawLine(beg_w,end_h,end_w,end_h);
	p.drawLine(beg_w,beg_h,beg_w,end_h);
	
	p.drawText(letter_width,beg_h-letter_height_up*2,y_axis);
	p.drawText(width()-letter_width*2*x_axis.size(), end_h+letter_height_down*2, x_axis);
	
	p.end();
}

