#include "historywidget.h"
#include "ar.h"

// patterns
// [lowest][level]<ind>
// [avg][level]<ind>
// [highest][level]<ind>
// [today][lowest]<ind>
// [today][avg]<ind>
// [today][highest]<ind>
// [ar][lowest][level]<ind>
// [ar][avg][level]<ind>
// [ar][highest][level]<ind>

bool historywidget::irc_satisfies_condition(int level,int i,QDateTime d_min,QDateTime d_max,bool utc)
{
	QDateTime d=QDateTime(QDate(h[level][i].year,h[level][i].month,h[level][i].day),
							QTime(h[level][i].hour,h[level][i].minute,h[level][i].second));
	if((utc && d.toUTC()<d_min) ||
			(!utc && d<d_min))
			return false;
	if((utc && d.toUTC()>d_max) ||
			(!utc && d>d_max))
			return false;
	return true;
}

double historywidget::irc_getlowest(formula* ind,int level,bool f,bool nf,QDateTime beg,QDateTime end,bool utc)
{
	bool no_date=beg.isNull();
	double res=0,cur;
	int i;
	
	cout<<"getlowest"<<endl;
	irc_level=level;irc_num=0;
	try
	{
		int num_games=0;
		for(i=0;i<h[level].size();++i)
			if((f || !h[level][i].flagging) && (nf || h[level][i].flagging))
				if(no_date || !irc_satisfies_condition(level,i,beg,end,utc))
				{
					irc_num=i;
					Eval.refresh(&(h[level][i]));
					res=Eval.eval(ind);
					++num_games;
					cout<<"game "<<i<<" of "<<h[level].size()<<endl;
					break;
				}
		for(++i;i<h[level].size();++i)
			if((f || !h[level][i].flagging) && (nf || h[level][i].flagging))
				if(no_date || !irc_satisfies_condition(level,i,beg,end,utc))
				{
					Eval.refresh(&(h[level][i]));
					if((cur=Eval.eval(ind))<res)
					{
						res=cur;
						irc_num=i;
					}
					++num_games;
					cout<<"game "<<i<<" of "<<h[level].size()<<endl;
				}
	}
	catch(exceval)
	{
		msg("Oh shit, it can not be, it can not be!");
	}
	return res;
}
double historywidget::irc_getavg(formula* ind,int level,bool f,bool nf,QDateTime beg,QDateTime end,bool utc)
{
	bool no_date=beg.isNull();
	double res=0;
	int i;
	int num_games=0;
	
	cout<<"getaverage"<<endl;
	try
	{
		for(i=0;i<h[level].size();++i)
			if((f || !h[level][i].flagging) && (nf || h[level][i].flagging))
				if(no_date || !irc_satisfies_condition(level,i,beg,end,utc))
				{
					Eval.refresh(&(h[level][i]));
					res+=Eval.eval(ind);
					++num_games;
					cout<<"game "<<i<<" of "<<h[level].size()<<endl;
				}
	}
	catch(exceval)
	{
		msg("Oh shit, it can not be, it can not be!");
	}
	return num_games ? res/num_games : 0;
}
double historywidget::irc_gethighest(formula* ind,int level,bool f,bool nf,QDateTime beg,QDateTime end,bool utc)
{
	bool no_date=beg.isNull();
	double res=0,cur;
	int i;
	
	cout<<"gethighest"<<endl;
	irc_level=level;irc_num=0;
	try
	{
		int num_games=0;
		for(i=0;i<h[level].size();++i)
			if((f || !h[level][i].flagging) && (nf || h[level][i].flagging))
				if(no_date || !irc_satisfies_condition(level,i,beg,end,utc))
				{
					irc_num=i;
					Eval.refresh(&(h[level][i]));
					res=Eval.eval(ind);
					++num_games;
					cout<<"game "<<i<<" of "<<h[level].size()<<endl;
					break;
				}
		for(++i;i<h[level].size();++i)
			if((f || !h[level][i].flagging) && (nf || h[level][i].flagging))
				if(no_date || !irc_satisfies_condition(level,i,beg,end,utc))
				{
					Eval.refresh(&(h[level][i]));
					if((cur=Eval.eval(ind))>res)
					{
						res=cur;
						irc_num=i;
					}
					++num_games;
					cout<<"game "<<i<<" of "<<h[level].size()<<endl;
				}
	}
	catch(exceval)
	{
		msg("Oh shit, it can not be, it can not be!");
	}
	return res;
}

void historywidget::irc_copy_records()
{
	QString s=recs.text();
	irc_process(s);
}

void historywidget::irc_copy_averages()
{
	QString s=avgs.text();
	irc_process(s);
}

void historywidget::irc_copy_ar()
{
	QString s=ar.text();
	irc_process(s);
}

void historywidget::irc_copy_today()
{
	if(today_beg.isChecked())
		irc_copy_today(0);
	else if(today_int.isChecked())
		irc_copy_today(1);
	else
		irc_copy_today(2);
}

void historywidget::irc_copy_today(int level)
{
	QString s=today.text();
	irc_process(s);
}

void historywidget::irc_process(QString s)
{
	QString res="";
	formula* ind=0;
	int cur=0,l,sz=s.size();
	double d=0.0;
	int level=2,prec=2;
	bool f=true,nf=true,utc=false;
	enum{low,avg,high} t=low;
	QDateTime ar_beg,ar_end;
	QDateTime today_beg,today_end;
	QDateTime null_date;
	QDateTime cur_beg,cur_end;
	init_ar(ar_beg,ar_end);
	init_today(today_beg,today_end);
	
	cout<<"irc_process"<<endl;
	try
	{
		while(cur<sz)
		{
			f=nf=true;
			cur_beg=cur_end=null_date;
			utc=false;
			t=low;
			
			// plain text part
			l=0;
			while(cur<sz && s[cur++] != '[') ++l;
			if(l) res += s.mid(cur-l-1,l);
			if(l) cout<<"ircprocess: plain text "<<s.mid(cur-l-1,l)<<endl;
			cout<<" cur="<<cur<<" l="<<l<<endl;
			--cur;
			
			while(s[cur]=='[')
			{
				// '[' found, search for pattern
				++cur;
				l=0;
				while(cur<sz && s[cur++] != ']') ++l;
				QString pattern=s.mid(cur-l-1,l).toLower();
				if(pattern=="lowest")
					t=low;
				else if(pattern=="average")
					t=avg;
				else if(pattern=="highest")
					t=high;
				else if(pattern=="f")
				{
					nf=false;f=true;
				}
				else if(pattern=="nf")
				{
					f=false;nf=true;
				}
				else if(pattern=="ar")
				{
					utc=true;cur_beg=ar_beg;cur_end=ar_end;
				}
				else if(pattern=="today")
				{
					utc=false;cur_beg=today_beg;cur_end=today_end;
				}
				else if(pattern=="beg")
					level=0;
				else if(pattern=="int")
					level=1;
				else if(pattern=="exp")
					level=2;
				else
					prec=pattern.toInt();
				
				cout<<" pattern '"<<pattern<<"'"<<endl;
				cout<<" cur="<<cur<<endl;
			}
			while(cur<sz && s[cur++]!='<');
			if(cur>=sz) break;
			
			// first formula
			l=0;
			while(cur<sz && s[cur++] != '>') ++l;
			cout<<"ircprocess: parsing formula "<<s.mid(cur-l-1,l)<<endl;
			if(ind) delete ind;
			ind=Parser.parse_formula(s.mid(cur-l-1,l).toLower()); 
			cout<<"ircprocess: parsed formula "<<ind->print()<<endl;
			cout<<" cur="<<cur<<endl;
		
			switch(t)
			{
				case low:
					d=irc_getlowest(ind,level,f,nf,cur_beg,cur_end,utc);
					break;
				case avg:
					d=irc_getavg(ind,level,f,nf,cur_beg,cur_end,utc);
					break;
				case high:
					d=irc_gethighest(ind,level,f,nf,cur_beg,cur_end,utc);
					break;
				default:
					d=0;
					break;
			}
			res += QString::number(d,'f',prec);
			cout<<"ircprocess: got result "<<d<<endl;
			cout<<" cur="<<cur<<endl; 
			
			while(1)
			{
				l=0;
				while(cur<sz && s[cur]!='<' && s[cur++]!='[') ++l;
				res += s.mid(cur-l-1,l);
				cout<<"ircprocess: plain text (inside formula) "<<s.mid(cur-l-1,l)<<endl;
				cout<<" cur="<<cur<<" l="<<l<<endl;
				
				if(cur>=sz) break;
				else if(s[cur-1]=='[')
				{
					if(s[cur]<'0' || s[cur]>'9') break;
					l=0;
					while(cur<sz && s[cur++] != ']') ++l;
					QString pattern=s.mid(cur-l-1,l).toLower();
					prec=pattern.toInt();
					cout<<"ircprocess: precision "<<prec<<endl;
					cout<<" cur="<<cur<<endl;
				}
				else
				{
					++cur;
					l=0;
					while(cur<sz && s[cur++] != '>') ++l;
					if(ind) delete ind;
					cout<<"ircprocess: parsing formula "<<s.mid(cur-l-1,l)<<endl;
					ind=Parser.parse_formula(s.mid(cur-l-1,l).toLower()); 
					Eval.refresh(&(h[irc_level][irc_num]));
					d=Eval.eval(ind);
					res += QString::number(d,'f',prec);
					cout<<"ircprocess: parsed formula "<<ind->print()<<endl;
					cout<<" cur="<<cur<<endl;
				}
			}
			if(cur>=sz) break; else --cur;
			
			cout<<"ircprocess: parsed "<<endl;
			cout<<" cur="<<cur<<endl;
		}
	}
	catch(exceval e)
	{
		msg("Evaluation error");
	}
	if(ind) delete ind;
	
	cout<<"ircprocess: result="<<res<<endl;
	
	QApplication::clipboard()->setText(res);
	
}

