#include "historywidget.h"

historywidget::historywidget():
S(settings_dir+"freesweeper-history.settings"),
Parser(&T),Eval(),f_x(0),f_y(0),cond(0),ind(0),d_min_utc(false),d_max_utc(false)
{
	hg.reset();
	cl.reset();
	ch.reset();

	place_things();

	read_history();
	
	g_exp_chosen();
	g_x_time_chosen();
	g_y_bbbvs_chosen();
	g_select_f.setChecked(true);
	g_select_nf.setChecked(true);
	t_select_f.setChecked(true);
	t_select_nf.setChecked(true);
	adj_x_axis_clicked();	
	adj_y_axis_clicked();	
	t_time_chosen();
	records.setChecked(true);
	
	connect_things();
	show_occurence();

	prec=2;
	a_step=1.0;
	
	S.read();
	move(S.getint("X"),S.getint("Y"));
	resize(S.getint("Width"),S.getint("Height"));
	recs.setText(S.get("IRCRecords"));
	avgs.setText(S.get("IRCAverages"));
	ar.setText(S.get("IRCAR"));
	today.setText(S.get("IRCToday"));
	tab.setCurrentIndex(S.getint("ActiveTab"));
	
	b_x_custom_edit.setText("BBBV");
	b_y_custom_edit.setText("BBBV");
	
	int ctrs=S.getint("CountersCount");
	for(int i=0;i<ctrs;++i)
	{
		QString pre="Counter"+QString::number(i);
		table_ctrs << Parser.parse_formula(S.get(pre));
		better_is_higher << S.getint(pre+"HigherIsBetter");
		table_headers << S.get(pre+"TableHeader");
		table_prec << S.getint(pre+"Precision");
		table_avg_prec << S.getint(pre+"AveragePrecision");
	}
	
	setWindowTitle("History");
}
historywidget::~historywidget()
{
	disconnect(&tab,SIGNAL(currentChanged(int)),this,SLOT(tab_changed(int)));
	hg.setParent(0);
	cl.setParent(0);
	ch.setParent(0);
	S.set("IRCRecords",recs.text());
	S.set("IRCAverages",avgs.text());
	S.set("IRCAR",ar.text());
	S.set("IRCToday",today.text());
	S.set("X",x());
	S.set("Y",y());
	S.set("Width",width());
	S.set("Height",height());
	S.set("CountersCount",table_ctrs.size());
	S.set("ActiveTab",tab.currentIndex());
	for(int i=0;i<table_ctrs.size();++i)
	{
		QString pre="Counter"+QString::number(i);
		S.set(pre,table_ctrs[i]->print());
		S.set(pre+"HigherIsBetter",better_is_higher[i]);
		S.set(pre+"TableHeader",table_headers[i]);
		S.set(pre+"Precision",table_prec[i]);
		S.set(pre+"AveragePrecision",table_avg_prec[i]);
	}
	S.write();
}

historywidget::historywidget(int nogui):S(settings_dir+"freesweeper-history.settings"),Parser(&T)
{
	nogui=nogui;
	read_history();
	S.read();
	recs.setText(S.get("IRCRecords"));
	avgs.setText(S.get("IRCAverages"));
	ar.setText(S.get("IRCAR"));
	today.setText(S.get("IRCToday"));
}

QString historywidget::date_to_string(int d,int m,int y)
{
	return QString::number(d).rightJustified(2, QLatin1Char('0'), true)+"."+
			QString::number(m).rightJustified(2, QLatin1Char('0'), true)+"."+
			QString::number(y).rightJustified(4, QLatin1Char('0'), true);
}
QString historywidget::date_to_string(QDate d)
{
	return QString::number(d.day()).rightJustified(2, QLatin1Char('0'), true)+"."+
			QString::number(d.month()).rightJustified(2, QLatin1Char('0'), true)+"."+
			QString::number(d.year()).rightJustified(4, QLatin1Char('0'), true);
}
QString historywidget::hour_to_string(int h,int m,int s)
{
	return QString::number(h).rightJustified(2, QLatin1Char('0'), true)+":"+
			QString::number(m).rightJustified(2, QLatin1Char('0'), true)+":"+
			QString::number(s).rightJustified(2, QLatin1Char('0'), true);
}
void historywidget::get_date_from_string(const QString& s,int& y,int& m,int &d)
{
	d=s.mid(0,2).toInt();
	m=s.mid(3,2).toInt();
	y=s.mid(6,4).toInt();
}

void historywidget::set_date_today()
{
	QDate d=QDate::currentDate();
	g_date_min.setText(date_to_string(d));
	g_date_max.setText(g_date_min.text());
	t_date_min.setText(g_date_min.text());
	t_date_max.setText(g_date_max.text());
	set_date_condition();
}
void historywidget::set_date_week()
{
	QDate d=QDate::currentDate();
	QDate d_first=d.addDays(1-d.dayOfWeek());
	QDate d_last=d.addDays(7-d.dayOfWeek());
	
	g_date_min.setText(date_to_string(d_first));
	g_date_max.setText(date_to_string(d_last));				
	t_date_min.setText(g_date_min.text());
	t_date_max.setText(g_date_max.text());
	set_date_condition();
}
void historywidget::set_date_month()
{
	QDate d=QDate::currentDate();
	d=d.addDays(1-d.day());
	g_date_min.setText(date_to_string(d));
	d=d.addDays(d.daysInMonth()-1);
	g_date_max.setText(date_to_string(d));				
	t_date_min.setText(g_date_min.text());
	t_date_max.setText(g_date_max.text());
	set_date_condition();
}
void historywidget::set_date_year()
{
	QDate d=QDate::currentDate();
	g_date_min.setText("01.01."+QString::number(d.year()));
	g_date_max.setText("31.12."+QString::number(d.year()));				
	t_date_min.setText(g_date_min.text());
	t_date_max.setText(g_date_max.text());
	set_date_condition();
}
void historywidget::set_date_ar()
{
	QDateTime d1,d2;
	init_ar(d1,d2);
	
	g_date_min.setText(date_to_string(d1.date())+" UTC");
	g_date_max.setText(date_to_string(d2.date())+" UTC");
	t_date_min.setText(g_date_min.text());
	t_date_max.setText(g_date_max.text());
	set_date_condition();
}

void historywidget::set_date_condition()
{
	int d,m,y;
	QString min=tab.currentIndex() ? t_date_min.text() : g_date_min.text();
	QString max=tab.currentIndex() ? t_date_max.text() : g_date_max.text();
	
	if(min.size()==10)
	{
		get_date_from_string(min,y,m,d);
		d_min=QDateTime(QDate(y,m,d),QTime(0,0,0));
		d_min_utc=false;
	}
	else if(min.size()==14 && min.right(4)==" UTC")
	{
		get_date_from_string(min,y,m,d);
		d_min=QDateTime(QDate(y,m,d),QTime(0,0,0));
		d_min_utc=true;
	}

	if(max.size()==10)
	{
		get_date_from_string(max,y,m,d);
		d_max=QDateTime(QDate(y,m,d),QTime(23,59,59));
		d_max_utc=false;
	}
	else if(max.size()==14 && max.right(4)==" UTC")
	{
		get_date_from_string(max,y,m,d);
		d_max=QDateTime(QDate(y,m,d),QTime(23,59,59));
		d_max_utc=false;
	}
	date_condition_on = tab.currentIndex()==0 ? g_select_date.isChecked() : t_select_date.isChecked();
	if(date_condition_on) refresh();
}
void historywidget::set_date_condition(QString& s)
{
	set_date_condition();
}

void historywidget::tab_changed(int)
{
	g_init_dates();
	t_init_dates();
	refresh();
}

void historywidget::refresh()
{
	if(tab.currentIndex()==0)
	{
		refresh_graph();
	}
	else if(tab.currentIndex()==1)
	{
		refresh_table();
	}
	else if(tab.currentIndex()==4)
	{
		show_density();
	}
}

bool historywidget::satisfies_condition(int i)
{
	return satisfies_condition(cur_level,i);
}
bool historywidget::satisfies_condition(int level,int i)
{
	if(!(h[level][i].flagging&&g_select_f.isChecked()) &&
		!(!h[level][i].flagging&&g_select_nf.isChecked()))
		return false;
	if(g_select_date.isChecked())
	{
		QDateTime d=QDateTime(QDate(h[level][i].year,h[level][i].month,h[level][i].day),
						QTime(h[level][i].hour,h[level][i].minute,h[level][i].second));
		if((d_min_utc && d.toUTC()<d_min) ||
			(!d_min_utc && d<d_min))
			return false;
		if((d_max_utc && d.toUTC()>d_max) ||
			(!d_max_utc && d>d_max))
			return false;
	}
	if(g_select_custom_on.isChecked() && cond)
	{
		Eval.refresh(&h[level][i]);
		return Eval.eval_expr(cond);
	}
	return true;
}

void historywidget::keyPressEvent(QKeyEvent* e)
{
	if(e->key()==Qt::Key_Enter || e->key()==Qt::Key_Return)
		if(tab.currentIndex()==2) makesearch();
}
void historywidget::closeEvent(QCloseEvent*)
{
	QCoreApplication::exit(0);
}

