#include "history.h"
#include "historywidget.h"

const QString box_header="FreeSweeper";

QString settings_dir,programs_dir;
char slash;

int main(int argc,char** argv)
{
	QApplication app(argc,argv);

#ifdef _WIN32
	settings_dir=QApplication::applicationDirPath()+"/";
	programs_dir=QApplication::applicationDirPath()+"/";
	slash='\\';
#else
	settings_dir=QDir::homePath()+"/.freesweeper/";
	programs_dir=QApplication::applicationDirPath()+"/";
	slash='/';
#endif

	if(argc==1)	// show history
	{
		historywidget hw;
		hw.show();
	
		return app.exec();
	}
	else if(argc==2)
	{
		historywidget hw(0);
		
		if(argv[1][0]=='r')	// records
			hw.irc_copy_records();
		else if(argv[1][0]=='a')	// avg
			hw.irc_copy_averages();
		else if(argv[1][0]=='A')
			hw.irc_copy_ar();
		else if(argv[1][0]=='b')
			hw.irc_copy_today(0);
		else if(argv[1][0]=='i')
			hw.irc_copy_today(1);
		else if(argv[1][0]=='e')
			hw.irc_copy_today(2);
		else
			return 1;
			
		return 0;
	}
	
	return 1;
}

ostream& operator<<(ostream& o,const QString& s)
{
	return o<<s.toStdString();
}

void msg(const QString& s)
{
	QMessageBox::information(0,box_header,s);
}
void err(const QString& s)
{
	QMessageBox::warning(0,box_header,s);
}
bool ask(const QString& s)
{
	return (QMessageBox::question(0,box_header,s,QMessageBox::Ok|QMessageBox::Cancel,QMessageBox::Cancel)==
		QMessageBox::Ok);
}
void crit_err(const QString& s)
{
	QMessageBox::critical(0,box_header,s+"\n\n Program will be terminated.");
}

QString input(QWidget* parent,const QString& query,const QString& def)
{
	return QInputDialog::getText(parent,box_header,query,QLineEdit::Normal,def);
}

int input_int(QWidget* parent,const QString& query,int def,int min,int max)
{
	return QInputDialog::getInteger(parent,box_header,query,def,min,max);
}

void writeln(QFile& f, const QString& s)
{
	const QByteArray endline("\n");
	f.write(s.toUtf8());
	f.write(endline);
}

QString readln(QFile& f)
{
	return QString::fromUtf8(f.readLine()).remove(QChar(10)).remove(QChar(13));
}

QString readln_without_comments(QFile& f)
{
	QString s=readln(f);
	while(s.size()>=2 && s[0]=='/' && s[1]=='/' && !f.atEnd()) s=readln(f);
	return s;
}

