#ifndef _FS_H_PARSER_H
#define _FS_H_PARSER_H

#include "history.h"
#include "formulaid.h"
#include "evaluable.h"
#include "translation.h"

struct excparser
{
	int sym;
	QString msg;
	excparser(int sym,const QString& msg):sym(sym),msg(msg){};
};

class parser
{
	QMap<QString,formula_id> name_to_id;

	inline QCharRef sym(QString& s, int cur);	
	
	int get_expression1(QString& s,int beg,formula*& f);
	// expression1 ::= expression2 or expression1 | expression2
	int get_expression2(QString& s,int beg,formula*& f);
	// expression2 ::= expression3 and expression2 | expression2
	int get_expression3(QString& s,int beg,formula*& f);
	// expression3 ::= [expression1] | formula1 < formula1 | formula1 > formula1 | formula1 = formula1
	int get_formula1(QString& s,int beg,formula*& f);	
	// formula1 ::= formula2 + formula1 | formula2 - formula1 | formula2
	int get_formula2(QString& s,int beg,formula*& f);
	// formula2 ::= formula3 * formula2 | formula3 / formula2 | formula3
	int get_formula3(QString& s,int beg,formula*& f);
	// formula3 ::= formula4 ^ formula3
	int get_formula4(QString& s,int beg,formula*& f);
	// formula4 ::= -formula4 | (formula1) | sqr(formula1) | log(formula1) | exp(formula1) | rank(int) | token 
	int get_token(QString& s,int beg,formula*& f);

	int get_function(QString& s,int& cur,formula*& f,formula_id x);
	
	void ignore_spaces(QString& s,int& cur);
	
	// token readers
	QString get_id(QString& s,int& beg);
	double get_number(QString& s,int& beg);
	int get_int(QString& s,int& beg);
public:
	translation* T;
	
	parser(translation* T);

	formula* parse_formula(QString s);
	formula* parse_expression(QString s);
};

#endif

